function SegmentCompartments(ibase, isuff, f0, f1, fwidth, h, fname)
% SegmentCompartments: SegmentCompartments(ibase, isuff, f0, f1, fwidth, h, fname)
% 
% Produces an amira surface file representation of the cell compartments in the stack.
%
%INPUT:
%------
% ibase - image base name whole path including file name up to the numerical 
%         counter for stack level.
% isuff - image suffix eg .tif
% f0    - start frame
% f1    - end frame
% fwidth - width of frame number for fixed-width formats eg. 3 for img001.jpg
% h      - half width of depth averaging in other words the number of frames
%          above and below the frame to average.  eg 1 means average one 
%          frame above and one frame below.
% fname  - the complete filename including path to save the amira surface text file
%
% OUTPUT:
%---------
% a text file with filename fname that is the amira surface file representation.
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% construct_fname.m
% median_filter.m
% icluster.m
% icluster_contours.m
% zlink.m
% cell_objects.m
% triangulate_cell_objects.m
% write_HxSurface.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------
ithr = 36;
minsize = 25;
maxsize = 20000;
n_steps = 5;
t = zeros(n_steps, 1);
fprintf (1, 'Starting SegmentCompartments\n');
step = 1;

tic
F = cell(f1-f0+1, 1);
ff = 0;
for f = f0:f1
  ff = ff+1;
  iname = construct_fname(ibase, isuff, f, fwidth);
  I = double(imread(iname));
  X = double(I < ithr);
  Y = median_filter(X, 5);
  [L, nobj] = icluster(Y, ones(2,1));
  A = icluster_moments(L, nobj);
  elim = find( (A(:,1) < minsize) | (A(:,1) > maxsize) ); 
  [L_new, nobj_new] = icluster_elim(L, nobj, elim);
  F{ff,1} = icluster_contours(L_new, nobj_new);
end
t(step) = toc;
fprintf (1, '  compartment stack done: %f\n', t(step)); 

step = step+1;
tic, [L, num_objects] = zlink(F); t(step) = toc;
fprintf (1, '  z link done: %f\n', t(step)); 

step = step+1;
tic, B = cell_objects(F, L, num_objects); t(step) = toc;
fprintf ('  cell ojects done: %f\n', t(step));

step = step+1;
tic, [V, T, C, L] = triangulate_cell_objects(B); t(step) = toc;
fprintf (1, '  triangulation done: %f\n', t(step));

step = step+1;
tic, write_HxSurface(fname, V, T, C, L); t(step) = toc;
fprintf (1, '  write_HxSurface done %f\n', t(step));

fprintf (1, 'Total Elapsed Time = %f\n', sum(t));
keyboard
