function C = bipartite_mutual_nn(arg1, arg2)
%
% BIPARTITE_MUTUAL_NN: C = bipartite_mutual_nn(D);
%                      C = bipartite_mutual_nn(X1, X2);
%
% Establish correspondences between points in X1 and X2 by
%   matching each point in X1 to its nearest neighbor in X2 and
%   matching each point in X2 to its nearest neighbor in X1. Only
%   matches that are "mutual" (meaning i matches to j and j matches to i)
%   are retained as valid correspondences.
%
% INPUT:
% -----
% With 1 input argument:
%   arg1 = D   - (N1 X N2) distance matrix
%
% With 2 input arguments:
%   arg1 = X1  - (N1 x d) matrix; each row is a point (say from first image)
%   arg2 = X2  - (N2 x d) matrix; each row is a point (say from second image)
% 
% In the case of 2 inout arguments, Euclidean distance between the input points is used.
%
% OUTPUT:
% ------
% C  - (M X 2) matrix fo correspondences; First column is the index into X1
%        and second column is the index into matching point in X2.
%
% HISTORY:
% -------
% 20070824 (MCB) - Modified to take either a distance matrix or two point sets. Also,
%   modified so that if there are ties for the nearest neighbor, each of the ties is 
%   considered to be a valid potential match.
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

if (nargin == 1)
  D = arg1;
  N1 = size(D,1);
  N2 = size(D,2);
elseif (nargin == 2)
  X1 = arg1;
  X2 = arg2;
  [N1, d1] = size(X1);
  [N2, d2] = size(X2);
  if (d2 ~= d1)
    fprintf(1, 'ERROR (bipartite_mutual_nn): dimensionality of points must be the same\n');
    C = [];
    return
  end
  D = distmat(X1, X2);
else
  fprintf(1, 'ERROR (bipartite_mutual_nn): number of input arguments must be 1 or 2\n');
  C = [];
  return
end

bj = zeros(N1,N2);
for i = 1:N1
  dmin = min(D(i,:));
  jmin = find(D(i,:) == dmin);
  for k = 1:length(jmin)
    bj(i, jmin(k)) = 1;
  end
end

bi = zeros(N1,N2);
for j = 1:N2
  dmin = min(D(:,j));
  imin = find(D(:,j) == dmin);
  for k = 1:length(imin)
    bi(imin(k),j) = 1;
  end
end

[I, J] = find(bj .* bi);
C = [I, J];

%k = 0;
%Ctmp = zeros(min(N1,N2), 2);
%for i=1:N1
%  if (bi(bj(i)) == i)
%    k = k+1;
%    Ctmp(k,:) = [i, bj(i)];
%  end
%end
%C = Ctmp(1:k,:);

