function [T1, T0] = code_delivery_system(build_dir, targets, flatten)
%
% CODE_DELIVERY_SYSTEM: [T1, T0] = code_delivery_system(build_dir, targets, flatten);
%
% Generate a parsimonious deliverable product for a specific project out of the burlcode 
% cvs repository by including targets and supporting dependencies and directory structure.
%
% INPUT:
% -----
% build_dir - char string; directory in which to create the deliverable hierarchy
%               this should be an existing directory with nothing in it.
% targets   - cell array with names of targets specified in burlcode hierarchy
% flatten   - [OPTIONAL] 0 = do not flatten (default), 1 = flatten
%               If flatten is selected, then the natural hierarchical structure of 
%               burlcode is eliminated. All *files* tagged for retention are placed
%               in the same directory under [build_dir, 'burlcode/']; all directory
%               structure is omitted. NAME COLLISIONS ARE SILENTLY IGNORED!
%               (e.g., if you have retained a  Makefile in one directory and a 
%               Makefile in another directory, but want to flatten everything into 
%               a single directory only one of these Makefiles will survive and no
%               warnings or errors will be issued) 
%               
%
% OUTPUT:
% ------
% T1        - directory tree showing directories and files included in the deliverable
% T0        - directory tree showing raw structure of burlcode hierarchy
%
% HISTORY:
% -------
% 20060819 (MCB) - incoming targets are now specified in burlcode hierarchy. We
%   append the build_dir to get full pathname. For example, one should now use:
%
%     targets{1,1} = src/programs/rockster/x_rockster.cpp; %(RIGHT)
% 
%   instead of
%
%     targets{1,1} = /tmp/delivery/burlcode/src/programs/rockster/x_rockster.cpp; %(WRONG)
%
% NOTE:
% ----
% With the non-flatten option, Makefiles and README files are automatically retained in 
% any directories that that are retained; however, the Makefiles will almost certainly
% require manual editing.
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% get_directory_tree.m
% find_dependencies.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------
if (nargin < 3)
  flatten = 0;
end

orig_dir = pwd;  % Note original directory
CVSROOT = '/proj/og1/repository/';
REQ_CHECKOUT = 1;
SHOW_DELETIONS = 0;
QUIET = 1;

%===================%
% CHECKOUT BURLCODE %
%===================%
if (REQ_CHECKOUT == 1)
  fprintf(1, 'Changing directory to %s\n', build_dir);
  cd(build_dir);
  fprintf(1, 'Checking out burlcode\n');
  cmd = ['!cvs -d ', CVSROOT, ' checkout -P burlcode'];
  eval(cmd);

  % Return to original directory
  cd(orig_dir);
end

%====================%
% GET DIRECTORY TREE %
%====================%
burlcode_dir = [build_dir, 'burlcode/'];
T0 = get_directory_tree(burlcode_dir);
n_nodes = size(T0,1);

%====================%
% TRACK DOWN TARGETS %
%   AND DEPENDENCIES %
%====================%

Q = zeros(n_nodes,1); % Flag vector for whether a node has been explored
A = zeros(n_nodes,1); % Flag vector for whether a node has been added to targets

% Convert initial target strings into full path
for k = 1:size(targets,1)
  targets{k,1} = [burlcode_dir, targets{k,1}];
end

% Convert intital target strings into node numbers
target_nodes = [];
for k = 1: size(targets,1)
  for j = 1:n_nodes
    if (strcmp(targets{k,1}, T0(j).fullname) == 1)
      A(j,1) = 1; % record that node j has been added to target_nodes      
      target_nodes = [target_nodes; j];
      break;
    end
  end
end

while (length(target_nodes) > 0)
  node = target_nodes(1);  
  if (Q(node,1) == 0)    % if node is unexplored, then explore it
    % Check for parent that has not been added
    if ((T0(node).parent ~= 0) & (A(T0(node).parent,1) == 0))
      % add the parent
      A(T0(node).parent,1) = 1;
      target_nodes = [target_nodes; T0(node).parent];
    end
 
    %----------------------------------------------------------------------------------------
    % If this node is a file, need to CHECK FOR DEPENDENCIES
    if (T0(node).isdir == 0)
      % Go through lines of file looking for includes, etc.
      if (~QUIET)
        fprintf(1, 'Checking %s for dependencies\n', T0(node).fullname);
      end
      H = find_dependencies(T0(node).fullname);

      % if any .h dependencies, need to also add in corresponding .c
      % if any .hpp dependencies, need to also add in corresponding .cpp      
      if (size(H,1) > 0)
        G = cell(size(H,1), 1);
	gcnt = 0;
        for h = 1:size(H,1)
	  fname = H{h,1};
	  L = length(fname);
	  if (L >= 3)
  	    if (strcmp(fname(L-1:L), '.h') == 1)
	      gcnt = gcnt + 1;
	      gname = [fname(1:L-2), '.c'];
	      G{gcnt,1} = gname;
	    end
	  end
	  if (L >= 5)
  	    if (strcmp(fname(L-3:L), '.hpp') == 1)
	      gcnt = gcnt + 1;
	      gname = [fname(1:L-4), '.cpp'];
	      G{gcnt,1} = gname;
	    end
	  end
        end
        if (gcnt > 0)
	  HH = cell(size(H,1) + gcnt);
	  for h = 1:size(H,1)
	    HH{h,1} = H{h,1};
	  end
	  for g = 1:gcnt
	    HH{size(H,1)+g,1} = G{g,1};
	  end
	  H = HH;
	end
      end

      % Determine node number for dependency h
      for h = 1:size(H,1)
	found = [];
	for j = 1:n_nodes
	  if (strcmp(T0(j).name, H{h,1}) == 1)
	    found = [found; j];
	  end
	end
	if (length(found) == 0)
	  fprintf(1, 'WARNING (code_delivery_system): unable to locate dependency %s\n', H{h,1});
	elseif (length(found) > 1)
          if (any(A(found,1) == 0))
  	    fprintf(1, 'WARNING (code_delivery_system): found multiple matches for dependency %s; adding all\n', H{h,1});
	    for l = 1:length(found)
	      fprintf(1, '  %s\n', T0(found(l)).fullname);
  	      if (A(found(l),1) == 0)
	        A(found(l),1) = 1;
	        target_nodes = [target_nodes; found(l)];
	      end
	    end
	  end
        else
	  % found exactly one node match; add it to targets
	  if (A(found(1),1) == 0)
	    A(found(1),1) = 1;
	    target_nodes = [target_nodes; found(1)];
	  end
	end
      end
    end
    %----------------------------------------------------------------------------------------      
    Q(node,1) = 1;
  end
  target_nodes(1) = [];  % remove this node from list of targets
end

%====================%
% MAKEFILES, ETC.    %
%====================%
if (flatten == 0)
  % For any directory marked for retention, see if it has a Makefile or README child
  for j = 1:n_nodes
    if ( (Q(j,1) == 1) & (T0(j).isdir == 1))
      for k = 1:length(T0(j).children)
	y = T0(j).children(k); % node number of k-th child
	cname = T0(y).name; % Get short name of k-th child
	if (strcmp(cname, 'Makefile') == 1)
	  Q(y,1) = 1;
	elseif (strcmp(cname, 'README') == 1)
	  Q(y,1) = 1;
	end
      end
    end
  end
end

%====================%
% DELETING FILES     %
%====================%

fprintf(1, 'Disposition of files from the %s hierarchy\n', T0(1).fullname)
if (SHOW_DELETIONS == 1)
  for j = 1:n_nodes
    if (Q(j,1) == 0)
      fprintf(1, 'DELETE: %s\n', T0(j).fullname);
    end
  end
end
fprintf(1, '******\n');
for j = 1:n_nodes
  if (Q(j,1) == 1)
    fprintf(1, 'RETAIN: %s\n', T0(j).fullname);    
  end
end
fprintf(1, '******\n');
fprintf(1, '\n\n');

proceed_flag = input('Proceed (0 = no, 1 = yes)? ');
if (proceed_flag == 0)
  fprintf(1, 'No action taken\n');
else
  if (flatten == 0)
    fprintf(1, 'System will begin deleting tagged files in 10 seconds\n');
    pause(10);
    fprintf(1, 'Last chance to hit CTRL-C to abort\n');
    pause(2);
    fprintf(1, 'Proceeding with deletion\n');  
  
    for j = 1:n_nodes
      if (Q(j,1) == 0)
	if (T0(j).isdir == 0)
	  cmd = ['!/bin/rm ', T0(j).fullname];
	else
	  cmd = ['!/bin/rm -r ', T0(j).fullname];	
	end
	D = dir(T0(j).fullname);
	if (size(D,1) > 0)
	  if (SHOW_DELETIONS == 1)
  	    fprintf(1, 'deleting %s\n', T0(j).fullname);
	  end
	  eval(cmd);
	end
      end
    end
    fprintf(1, 'Deletion completed!\n');  
  else
    fprintf(1, 'System will begin flattening in 10 seconds\n');
    pause(10);
    fprintf(1, 'Last chance to hit CTRL-C to abort\n');
    pause(2);
    fprintf(1, 'Proceeding with flattening\n');  
    
    % Create a temporary directory under build_dir and copy retained *files* there
    flattened_dir = [build_dir, 'flattened/'];
    cmd = ['!mkdir ', flattened_dir];
    eval(cmd);
    pause(2);
    cd(flattened_dir);
    
    for j=1:n_nodes
      if (Q(j,1) == 1)
	if (T0(j).isdir == 0)
	  cmd = ['!cp ', T0(j).fullname, ' .', ];
	  eval(cmd);
	end
      end
    end

    % remove burlcode hierarchy
    cmd = ['!/bin/rm -r ', burlcode_dir];
    eval(cmd);

    % move flattened dir to burlcode
    cmd = ['!/bin/mv ', flattened_dir, ' ', burlcode_dir];
    eval(cmd);
    
    cd(orig_dir);
    fprintf(1, 'Flattening completed!\n');      
  end
end

T1 = get_directory_tree(burlcode_dir);
