function Pset = compartments_from_graph(V, L)
%
% COMPARTMENTS_FROM_GRAPH: Pset = compartments_from_graph(V, L);
%
% INPUTS:
% ------
% V - (n_vertices X 2); coordinates of vertices (nodes) in graph
% L - (n_links X 2); links specify indices of nodes that are linked together
%       note: links are undirected.
%
% OUTPUT:
% ------
% Pset - cell array with one cell for each comartment found. The cell contains
%          ordered list of nodes going counterclockwise around compartment
%
% NOTE:
% ----
% No vertex should be repeated in V; also, a vertex should not link to itself.
% Dangling links can lead to some degenerate or partially degenerate
% compartments, as well as some potential redundancies, but otherwise should be ok.
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

% Eliminate any self-links
sl = find(L(:,1) == L(:,2));
if (~isempty(sl))
  L(sl,:) = [];
fprintf(1, 'WARNING (compartments_from_graph): eliminated %d self-links\n', length(sl));  
end


% Given links find 1-hop neighbors of each node
n_vertices = size(V,1);
N = cell(n_vertices, 1);
cnt = 0;
for v = 1:n_vertices
  ind1 = find(L(:,1) == v);
  ind2 = find(L(:,2) == v);
  N{v,1} = unique([L(ind1, 2);  L(ind2, 1)]);
  cnt = cnt + length(N{v,1});
end

% Consider each vertex as a starting point
Pset = cell(cnt,1); % loose upper bound on number of potential compartments
cc = 0;
for v = 1:n_vertices
  Nv = N{v,1};
  for k = 1:length(Nv) % Possible exits from vertex v
    redundant = 0; % Start out assuming not redundant

    w = Nv(k,1);
    P = [v; w]; % start of path around compartment
    Pvec = (V(w,:) - V(v,:))';
    Pvec = Pvec/norm(Pvec);
    done = 0;
    while (done == 0)
      Nw = N{w, 1};
      lNw = length(Nw);
      theta = zeros(lNw, 1);
      % Sort exit paths from w by angle wrt Pvec
      for l = 1:lNw
	x = Nw(l,1);
	Qvec = (V(x,:) - V(w,:))';
	if (norm(Qvec) == 0)
	  fprintf(1, 'Qvec\n');
	  keyboard
	end
	Qvec = Qvec/norm(Qvec);
	cth = Pvec'*Qvec;	
	zsth = cross([Pvec; 0], [Qvec; 0]);
	theta(l,1) = atan2(zsth(3,1), cth);
	if (x == P(size(P,1)-1,1))
	  % dont allow going directly back to previous point
	  theta(l,1) = -Inf;
	end
      end
      [theta_sort, ind_sort] = sort(theta, 1, 'descend');
      x_best = Nw(ind_sort(1));
      if (x_best == v)
	done = 1;
      else
	% Add a segment to compartment path
        P = [P; x_best];
	
	% Perform redundancy check
	if (size(P,1) == 3)
	  for m = 1:cc
	    Pm = [Pset{m,1}; Pset{m,1}(1:2)];
	    ind = find(P(1,1) == Pm);
	    if (~isempty(ind))
	      for j = 1:length(ind)
		if (ind(j)+2 <= size(Pm,1))
	 	  if (all(P(1:3) == Pm(ind(j):ind(j)+2)))
		    redundant = 1;
		    break;
		  end
		end
	      end
	    end
	    if (redundant == 1)
	      break;
	    end
	  end
	end
	
	if (redundant == 1)
	  done = 1;
	else
          Pvec = (V(x_best,:) - V(w,:))';
	  if (norm(Pvec) == 0)
	    fprintf(1, 'Pvec\n');	    
	    keyboard
	  end
          Pvec = Pvec/norm(Pvec);
          w = x_best;
	end
      end	
    end
    if (redundant == 0)
      cc = cc+1;
      Pset{cc,1} = P;
    end
  end
end

% Squish Pset down to minimum size
Ptmp = Pset;
Pset = cell(cc,1);
for i=1:cc
  Pset{i,1} = Ptmp{i,1};
end
