function [d, V1, V2] = contour_distance(C1, C2, M)
% CONTOUR_DISTANCE: d = contour_distance(C1, C2, M);
%
% Compute the distance between two **closed** contours
%
% INPUT:
% -----
% C1          - (n1 x 2) matrix of points on a contour
% C2          - (n2 x 2) matrix of points on a contour
% M           - number of points in resampling of contours
%
% OUTPUT:
% ------
% d           - scalar measuring "distance" between contours
% V1          - resampled contour C1
% V2          - resampled contour C2 (shifted and if necessary flipped to align with V1)
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% arcresamp.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

if (nargin <= 2)
  M = 50;
end

% Determine length of each contour
x1 = C1(:,1);
y1 = C1(:,2);
delta1 = [0; sqrt((diff(x1)).^2 + (diff(y1)).^2)];
l1 = sum(delta1);

x2 = C2(:,1);
y2 = C2(:,2);
delta2 = [0; sqrt((diff(x2)).^2 + (diff(y2)).^2)];
l2 = sum(delta2);

% Resample each contour to get same number of points on each
%   Go around contour twice to avoid end effects.
ds1 = l1/M;
ds2 = l2/M;
U1 = arcresamp([C1; C1], ds1, 'linear');
U2 = arcresamp([C2; C2], ds2, 'linear');

n1 = size(U1, 1);
n2 = size(U2, 2);
if (n2 >= 2*M)
  fprintf(1, 'ERROR (contour_distance): not enough points in U2\n');
  d = Inf;
  return
end

% Get RMS distance between contours assuming various offsets in start point
%   Also need to check for different sense (clockwise vs counterclockwise)
e = zeros(2*M,1);
for m = 1:M
  e(m,1) = sqrt(sum(sum((U1(1:M,:) - U2(m:m+M-1,:)).^2))/M);
end
for m=1:M
  e(M+m,1) = sqrt(sum(sum((U1(1:M,:) - U2(m+M-1:-1:m,:)).^2))/M);
end

% Report the minimum
[dmin, ind] = min(e);

d = dmin(1);

if (nargout > 1)
  V1 = U1(1:M,:);
  if (ind(1) <= M)
    m = ind(1);
    V2 = U2(m:m+M-1,:);
  else
    m = ind(1)-M;
    V2 = U2(m+M-1:-1:m,:);    
  end
end

