function K = curvature(e, ds, w);
%
% CURVATURE: K = curvature(e, ds, w);
%
% INPUT:
% -----
% e   - (N X 2) matrix containing (x,y) points of a curve;
%         *ALREADY* uniformly resampled according to arc length
% ds    - arc length sample spacing (OPTIONAL)
% w     - estimate dtheta from points w samples behind and w samples ahead
%
% OUTPUT:
% ------
% K     - curvature at each point (a vector the same size as x and y)
%
% SEE ALSO:
% --------
% boundary_resample.m
% curve_resample.m
%
% NOTE:
% ----
% Method used here is different than the technique used in the 
% curvature_old.m program, but should be a valid estimate of dtheta/ds.
%
% HISTORY:
% -------
% 2005/11/17 (MCB) - Introduced w; need N at least 2*w+1
% 2006/06/27 (MCB) - Can specify ds as Inf to force estimation of the value
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------
if (nargin < 3)
  w = 1;
end
  
N = size(e,1);
K = zeros(N,1);
if (N < 2*w+1)
  % Cannot do anything with such a short segment
  return
end

x = e(:,1);
y = e(:,2);

%% Apply pre-smoothing to contours if desired
% pre_smooth = 3;
% x = gsmooth_1d(x, pre_smooth, 'hold');
% y = gsmooth_1d(y, pre_smooth, 'hold');

dx = diff(x);
dy = diff(y);

% If arc length sample spacing NOT provided, estimate the value
if ((nargin < 3) | (isinf(ds)))
  s = sqrt(dx.^2 + dy.^2);
  ds = median(s);
end

for n = w+1:N-w
  vb = [x(n)-x(n-w); y(n)-y(n-w)];
  vf = [x(n+w)-x(n); y(n+w)-y(n)];

  % Compute the curvature
  dtheta = atan2(vb(1)*vf(2)-vb(2)*vf(1), vb(1)*vf(1)+vb(2)*vf(2));
%  dtheta = atan2(dx(n-1)*dy(n)-dy(n-1)*dx(n), dx(n-1)*dx(n)+dy(n-1)*dy(n));
  K(n,1) = dtheta/(w*ds);
end

% Guess values for first and last points since derivative
%   estimates fail at these points
K(1:w) = K(w+1);
K(N-w+1:N) = K(N-w); %% 20060804 (MCB) - RHS used to say K(N-w+1)


