function Z = draw_polygon(pc, lc);
%
% DRAW_POLYGON: Z = draw_polygon(pc, lc);
%
% Draw a polygon on a figure using mouse clicks and return
% vertices in Z. The mouse buttons (MB) are defined as follows:
%   MB1: ENTER a point
%   MB2: ERASE last point from the polygon
%   MB3: QUIT and close the polygon
%
% Note: The last click (MB3) is not included in the polygon.
%
% INPUT:
% -----
% pc = point code ('ro' = default)
% lc = line code  ('y-' = default)
%
% OUTPUT:
% ------
% Z contains 4 columns and (N+1) rows where N is the number of vertices.
% Z(:,1) = x-coords
% Z(:,2) = y-coords
% Z(:,3) = object handles for the points drawn on the figure
% Z(:,4) = object handles for the lines drawn on the figure
%
% CAUTION:
% -------
% There is a quirk in MATLAB that causes me a problem. When you get input
% with the mouse, the object over which you click gets moved to the top
% of the Children list. This means when things are redrawn, the image 
% is drawn on top of some of the polygon lines and under others. To fix
% this, I plot the polygon lines and points in 3-D at a constant z-value
% (positive) so all lines and points are drawn on top of the image.
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

if (nargin < 2)
  pc = 'ro';
  lc = 'y-';
end
if (length(lc) < 2)
  lc = [lc, '-'];
end

hold('on');

% Define mouse button functions
ENTER = 1;
ERASE = 2;
QUIT  = 3;

% Constant z-value so polygon appears on top of image
z = 0.1;
Z = [];

button = ENTER;
while (button ~= QUIT)
  npts = size(Z,1);
  [x, y, button] = ginput(1);
  if (button == ENTER)
    % Draw symbol at the point getting handle
    h1 = line(x, y, z, 'Marker', pc(1,2), 'Color', pc(1,1));
    if (npts > 0)
      % Draw line between this point and previous
      h2 = line([Z(npts,1), x], [Z(npts,2), y], [z z], 'Color', lc(1,1), 'Linestyle', lc(1,2));
      Z = [Z; [x y h1 h2]];
    else
      Z = [x y h1 -1];
    end
  elseif (button == ERASE)
    if (npts > 0)
      delete(Z(npts, 3));   
      delete(Z(npts, 4));
      Z(npts,:) = [];
    end
  end
end


% Close up the polygon by adding the first point to end of Z
npts = size(Z,1);
if (npts < 3)
  error('Need at least 3 points to define a polygon');
else
  x = Z(1,1);
  y = Z(1,2);
  h1 = line(x, y, z, 'Marker', pc(1,2), 'Color', pc(1,1));
  h2 = line([Z(npts,1), x], [Z(npts,2), y], [z, z], 'Color', lc(1,1), 'Linestyle', lc(1,2));
  Z = [Z; [x y h1 h2]];
end