function S = edge_detection(I, smooth, hi_thr, lo_thr)
%
% EDGE_DETECTION: S = edge_detection(I, smooth, hi_thr, lo_thr)
%
% Detect edges in an image using gradient magnitude, nonmax suppression,
% hysteresis thresholding, and linking (similar to Canny).
%
% INPUT:
% -----
% I      - image
% smooth - non-negative integer specifying number of smoothing passes 
%             w/3-tap binomial kernel
% hi_thr - fraction of maximum gradient magnitude for starting an edge (def = 0.2)
% lo_thr - fraction of maximum gradient magnitude for continuing an edge (def = 0.05)
%
% OUTPUT:
% ------
% S  - cell array of edge segments, S{i,1} is an n(i) X 2 array of points in
%        segment i. FIrst col is x, second col is y.
%
% HISTORY:
% -------
% 20040923 - cells in S are now returned in (x,y) format rather than (row,col)
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% ggradient.m
% nonmax_suppression.m
% edge_linker.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

if (nargin < 4)
  hi_thr = 0.2;
  lo_thr = 0.05;
end

[Mag, Theta] = ggradient(I, smooth, 1);
M = nonmax_suppression(Mag, Theta);

MMag = M.*Mag;
mmax = max(max(MMag));
t1 = hi_thr * mmax;
t0 = lo_thr * mmax;

S = edge_linker(MMag, t1, t0);

% Change reporting method to return [x,y] instread of [row,col]
for i = 1:size(S,1)
  s = S{i,1};
  S{i,1} = [s(:,2), s(:,1)];
end
