function S = edge_linker(I, hi_thr, lo_thr)
%
% EDGE_LINKER: S = edge_linker(I, hi_thr, lo_thr)
%
% This function takes the gradient magnitude image after it has been
% masked with the nonmax_suppression and links on-pixels into segments.
% Hysteresis thresholding is applied so that segments are only started
% from points that exceed hi_thr.
%
% INPUT:
% -----
% I - gradient magnitude image
% hi_thr - threshold for starting a contour
% lo_thr - threshold for continuing once you're on a contour
%
% OUTPUT:
% ------
% S - cell array (n_segments x 1),  S{i,1} will be an n(i) x 2 array
%       of the n(i) points in segment i. First col is the row coord; second 
%       col is the col coord.
%
% NOTE: For start pixels with more than two neighbors (e.g., at a junction 
% of three lines), the linker makes an arbitrary decision to splice two of the
% outgoing segments together, while the third is treated as a separate segment.
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% follow_segment.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------


% Embed I in an image so that we have a zero border around outside
[ir,ic] = size(I);
II = zeros(ir+2,ic+2);
II(2:ir+1, 2:ic+1) = I;

[r,c] = find(II >= hi_thr); % potential start pixels
JJ = (II >= lo_thr);    % binary image of points above lo_thr
smax = length(r); % Conservative estimate of maximum number of possible segments
Stmp = cell(smax, 1);    


L = zeros(ir+2, ic+2);

label = 0;
for k = 1:smax 
  if (L(r(k),c(k)) == 0) 
    % if it hasn't been labeled already, follow_segment
    label = label + 1;
    start_point = [r(k), c(k)];
    [s, L] = follow_segment(start_point, JJ, L);
    Stmp{label,1} = (s - ones(size(s,1),2)); % adjust for the embedding of I into II
    
    % Try to follow_segment again from the same startpoint in case we started
    %   in middle of a segment
    [s, L] = follow_segment(start_point, JJ, L);
    ns = size(s,1);
    if (ns > 1)
      % Eliminate redundant startpoint,adjust for embedding and combine
      Stmp{label,1} = [s(ns:-1:2,:)-ones(ns-1,2); Stmp{label,1}]; 
    end
  end
end

% Compress cell array down to minimum size needed
% Need to corret for the addition of border
S = cell(label,1);
for l = 1:label
  S{l,1} = Stmp{l,1};
end

