function [Lnew, XYnew] = edit_links(L, XY, I)
%
% EDIT_LINKS: [Lnew, XYnew] = edit_links(L, XY, I);
%
% This function allows one to edit a 2D graph using the mouse. The initial graph is
% given as a set of vertices XY and links L between various vertices. Mouse buttons are
% used to add or remove vertices and add or remove links.
%
% INPUT:
% -----
% L  - (n_links X 2); first column has index of start vertex; 2nd col has index of end vertex
% XY - (n_vertices X 2); 2D coordinates of vertices
% I  - grayscale image 
%
% OUTPUT:
% ------
% Lnew - new matrix of links
% XYnew -new matrix of vertices
%
% NOTES:
% -----
% Button1: Modify links 
% Button2: Modify vertices
% Button3: exit
%
% This function can be used to manually correct the set of links automatically established
% by link_vertices.m.
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% fig.m
% hon.m
% distmat.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------
dthr = 5;
Lnew = L;
XYnew = XY;

fig(I); hon;
n_vertices = size(XYnew,1);
hv = zeros(n_vertices,1);
for n = 1:n_vertices
  hv(n,1) = plot(XYnew(n,1), XYnew(n,2), 'bo');
end
gaxis;

n_links = size(L,1);
hl = zeros(n_links, 1);
for n = 1:n_links
  ii = Lnew(n,1);
  jj = Lnew(n,2);
  hl(n) = plot([XYnew(ii,1), XYnew(jj,1)], [XYnew(ii,2), XYnew(jj,2)], 'r', 'Linewidth', 2);
end

done = 0;
while (done == 0)
  [u1, v1, button1] = ginput(1);
  if (button1 == 3)
    done = 1;
  else
    % Determine if a node was picked and which one
    d = distmat([u1,v1], XYnew);
    [dmin1, imin1] = min(d);
    if (length(imin1) > 1)
      fprintf(1, 'length(imin1) = %d\n', length(imin1));
      keyboard
    end
    
    if (button1 == 2) % Add or remove a vertex
      % Does a vertex already exist at this point
      if (dmin1 > dthr)
	% Create a new vertex
	XYnew = [XYnew; [u1, v1]];
	hv0 = plot(u1, v1, 'co');
	hv = [hv; hv0];
      else
	% Remove old vertex and all its links
	delete(hv(imin1));
	hv(imin1) = [];
	XYnew(imin1,:) = [];

	rot = find((Lnew(:,1) == imin1) | (Lnew(:,2) == imin1));
	delete(hl(rot));
	hl(rot) = [];
	Lnew(rot,:) = [];
	
	% Need to adjust any existing links that have indices to points in
	%  XYnew that are above imin1
        ind1 = find(Lnew(:,1) > imin1);
	Lnew(ind1,1) = Lnew(ind1,1)-1;
        ind2 = find(Lnew(:,2) > imin1);
	Lnew(ind2,2) = Lnew(ind2,2)-1;
      end
    else % Add or remove a link
      [u2, v2, button2] = ginput(1);
      if (button2 == 3)
        done = 1;
      else
        % Determine which node was picked second
        d = distmat([u2,v2], XYnew);
        [dmin2, imin2] = min(d);
        if (length(imin2) > 1)
          fprintf(1, 'length(imin2) = %d\n', length(imin2));
          keyboard
        end
      
        % Does a link already exist between these
        f = find( ((Lnew(:,1) == imin1) & (Lnew(:,2) == imin2)) | ((Lnew(:,1) == imin2) & (Lnew(:,2) == imin1)) );
	if (~isempty(f))
	  % Delete existing link
	  delete(hl(f));
	  hl(f) = [];
	  Lnew(f,:) = [];
        else
	  % Add new link
	  Lnew = [Lnew; [imin1, imin2]];
	  hl0 = plot([XYnew(imin1,1), XYnew(imin2,1)], [XYnew(imin1,2), XYnew(imin2,2)], 'g', 'Linewidth', 2);
	  hl = [hl; hl0];
        end
      end
    end
  end
end
