function [R_master, Cmap] = extract_hand_truth(min_cnt)
%
% EXTRACT_HAND_TRUTH: R_master = extract_hand_truth(min_cnt)
%
% Extract hand labeled ground truth from images provided by Adrienne Roeder. Ground
%   truth is in the form of constant color/intensity disks where the color represents
%   an object ID. The disks are superimposed on a false-colored projection image, which
%   complicates the process. Also, disks may be touching or overlapping.
%
% INPUT:
% -----
% min_cnt       - scalar; minimum pixel count for any connected component.
%
% OUTPUT:
% ------
% R_master      - (N x 4) matrix with hand truth information for each nucleus that
%                   appears any time during the duration of the 4D stack sequence
%
%                   col1 - timepoint
%                   col2 - x centroid
%                   col3 - y centroid
%                   col4 - nucleus id
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% icluster_moments.m
% icluster_keep.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

if (nargin < 1)
  min_cnt = 120;
end

%% FLOWER_5
%idir = '/proj/og1/biology/aroeder/data/flower_5/hand_tracked/dots_only/';
%ibase = '5_';
%isuff = 'zdots.tif';
% N = 17;

% FLOWER_6
idir = '/proj/og1/biology/aroeder/data/flower_6/hand_tracked/dots_only/';
ibase = '6_';
isuff = 'dots.tif';
N = 7;

C_master = [];
R_master = [];
for n = 1:N
%  if ((n ~= 10) & (n ~= 16))
    iname = construct_fname([idir, ibase], isuff, n, 0);
    RGB = imread(iname);
    I = mean(RGB, 3);
    D = bwdist(~(I < 255)); % I has 255=white background
    L0 = watershed(255 - round(scale(D, 0, Inf, 0,255)));
    L0 = L0 .* (I < 255);
    nobj0 = max(max(L0));
    A0 = icluster_moments(L0, nobj0);
    keep = find(A0(:,1) > min_cnt);
    [L, nobj] = icluster_keep(L0, nobj0, keep);
    A = A0(keep, :);

    % Extract colors associated with each nucleus; eventually convert these colors into a
    %   globally-consistent ID for each nucleus.
    C = zeros(nobj, 3);
    for i = 1:nobj
      C(i,:) = RGB(round(A(i,3)), round(A(i,2)), :);
    end
    
    R = [n * ones(nobj,1), A(:,2), A(:,3), zeros(nobj,1)]; % [n, x, y, 0] for each nuclei

    C_master = [C_master; C];
    R_master = [R_master; R];
%  end
end

[B, ii, jj] = unique(C_master, 'rows'); % Find the unique colors in C_master
R_master(:,4) = jj;
Cmap = B/255;
DISPLAY_FIGURES = 1;

keyboard
if (DISPLAY_FIGURES == 1)
  % DISPLAY FINAL RESULTS
  for n = 1:N
%    if ((n ~= 10) & (n ~= 16))
      iname = construct_fname([idir, ibase], isuff, n, 0);
      RGB = imread(iname);
      figure;
      image(RGB);hon;
      title(int2str(n));
      ind = find(R_master(:,1) == n);
      for l = 1:length(ind)
	k = ind(l);
	text(R_master(k,2), R_master(k,3), int2str(R_master(k,4)), 'Color', 'k', 'Fontsize', 12);
      end
    end
%  end
end
keyboard
