function Pn = facets_to_normals(Pf, right_hand)
%
% FACETS_TO_NORMALS: Pn = facets_to_normals(Pf, right_hand);
%
% Convert polyhedra representation from facets to normals. In the facets representation,
% the bounding polygon for each facet is given. In the normals representation, which
% can only be used for convex polyhedrons, the outward surface normal (unit vector)
% and distance from the origin along the normal direction is given for each facet. A point
% [x;y;z] on the facet satisfies A*[x;y;z;-1] = 0.
%
% INPUT:
% -----
% Pf - (n_cells x 1) cell array; each cell contains a facets representation for a single polyhedron
%
% OUTPUT:
% ------
% Pn - (n_cells x 1) cell array; each cell contains a normals representation for a single polyhedron
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

if (nargin < 2)
  right_hand = 1; % assume right hand rule applied to facet contours yields outward normal 
end
n_cells = size(Pf, 1);
Pn = cell(n_cells, 1);

for n = 1:n_cells
  F = Pf{n,1};
  n_facets = size(F,1);
  A = zeros(n_facets, 4); % First three cols are outward facing unit normals
  for f = 1: n_facets
    V = F{f,1};
    if (size(V,1) < 3)
      fprintf(1, 'ERROR (facets_to_normal): degenerate facet\n');
      Pn = [];
      return
    end
    
    % Bruce's data seems to follow the left hand rule, so in his
    %   case we need to negate the normal to get outward facing.
    if (right_hand == 0)
      nvec = -(cross(V(2,:)-V(1,:), V(3,:)-V(1,:)))';
    else
      nvec = (cross(V(2,:)-V(1,:), V(3,:)-V(1,:)))';      
    end
    
    nvec_hat = nvec/norm(nvec);
    b = V(1,:) * nvec_hat;
    A(f,:) = [nvec_hat', b];
  end
  Pn{n,1} = A;
end

keyboard