function [V_master, T_master, C_master, L_master] = facets_to_triangulated(P, C)
%
% FACETS_TO_TRIANGULATED:  [V_master, T_master, C_master, L_master] = facets_to_triangulated(P, C);
%
% Given a collection of polyhedrons in facet form in P, generate a
% triangulated surface representation; the optional input matrix C
% gives an RGB color to each polyhedron.
%
% INPUT:
% -----
% P - (n_cells X 1) cell array of polyhedrons in facet form; 
%        each cell contains collection of facets
% C - (n_cells X 3) color matrix [optional]
%
% OUTPUT:
% ------
% V_master  - (nv x 3) matrix of 3D vertices
% T_master  - (nt x 3) matrix of triangles (each row contains 3 indices into vertices)
% C_master  - (nt x 3) matrix of RGB colors for each triangle
% L_master  - (nt x 1) matrix of object IDS
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% randcol.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

n_cells = size(P,1);
if (nargin < 2)
  C = randcol(n_cells, 0.3);
end

V_master = [];
T_master = [];
C_master = [];
L_master = [];

for n = 1:n_cells
  F = P{n,1};
  n_facets = size(F,1);
  for f = 1:n_facets
    Vf = F{f,1};
    M = size(Vf,1);
    mu = mean(Vf);
    Vf = [Vf; mu];
    Tf = [(M+1) * ones(M,1), (1:M)', [(2:M)'; 1]];
    vr = size(V_master,1);
    V_master = [V_master; Vf];
    T_master = [T_master; vr*ones(M,3)+Tf];  % Need to adjust vetrex indices to the master vertex array
    C_master = [C_master; ones(M,1) * C(n,:)];
    L_master = [L_master; n * ones(M,1)];
  end
end