function M = flower_5_growth()
%
% FLOWER_5_GROWTH: M = flower_5_growth()
%
% Use 3D nucleus position data and lineage labels for flower_5 to derive principal
% growth rates versus time.
%
% INPUT:
% -----
%
% OUTPUT:
% ------
% M - (n_use-1 X 3) matrix; n_use is the number of usable stack frames (time points).
%       M records a principal growth rate along three orthogonal axes between each 
%       adjacent pair of stack frames (time points).
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% combined_tll.m
% lineage_label_match.m
% estimate_affine_transformation.m
% hon.m
% gron.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

% scalings to convert voxel coords into metric coords
D = [1.80, 1.80, 2;
     0.90, 0.90, 2;
     0.90, 0.90, 2;
     0.90, 0.90, 2;
     0.90, 0.90, 2;
     0.90, 0.90, 2;
     0.90, 0.90, 2;
     0.90, 0.90, 2;
     1.27, 1.27, 2;
     1.27, 1.27, 2;
     1.27, 1.27, 2;
     1.27, 1.27, 2;
     1.27, 1.27, 2;     
     1.27, 1.27, 2;     
     1.27, 1.27, 2;     
     1.80, 1.80, 2;          
     2.54, 2.54, 2];
 
[T_master, LL_master] = combined_tll();

use = [1:9,11:15,17]';  % usable stack frames (omit 10 and 16)
n_use = length(use);

M = zeros(n_use-1, 3);
for i = 1:(n_use-1)
  j = use(i);
  k = use(i+1);
  
  indj = find(T_master(:,1) == j);
  indk = find(T_master(:,1) == k);
  
  XYZj_voxel = T_master(indj,2:4);
  LLj  = LL_master(indj,1);
  
  XYZk_voxel = T_master(indk,2:4);
  LLk  = LL_master(indk,1);
  
  % scale voxel coords into metric coords
  Dj = diag(D(j,:));
  Dk = diag(D(k,:));
  XYZj_metric = XYZj_voxel * Dj;
  XYZk_metric = XYZk_voxel * Dk;

  % Find correspondences
  C = lineage_label_match(LLj, LLk);
  
  % Keep all correspondences
  keep = logical(ones(size(C,1),1));
  
  % Corresponding points
  XYZjc = XYZj_metric(C(keep,1),:);
  XYZkc = XYZk_metric(C(keep,2),:);
  
  A = estimate_affine_transformation(XYZjc, XYZkc);
  B = A(1:3, 1:3);
  [U,S,V] = svd(B,0);
  M(j,:) = diag(S)';
end
      
figure;
hr = 12*(use-1); % time in hours
g1 = cumprod([1;M(use(1:(n_use-1)),1)]);
g2 = cumprod([1;M(use(1:(n_use-1)),2)]);
g3 = cumprod([1;M(use(1:(n_use-1)),3)]);
plot(hr,g1,'r','Linewidth',2);
hon;
plot(hr,g2,'g','Linewidth',2)
plot(hr,g3,'b','Linewidth',2)

% from flower_6
%% add best fit for the new curves
%alpha = 1.024;
%beta  = 1.013;
%plot(hr, alpha.^hr, 'g');
%plot(hr, beta.^hr, 'g');

title('Flower\_5 Growth', 'Fontsize', 18);
xlabel('elapsed time (hours)', 'Fontsize', 18);
gron
keyboard
