function [M, R] = flower_6_growth()
%
% FLOWER_6_GROWTH: [M, R] = flower_6_growth()
%
% Estimate principal growth rates in 2D flower_6 data by applying SVD to the estimated
% affine transformation between corresponding points.
%
% INPUT:
% -----
%
% OUTPUT:
% ------
% M - (n_frames-1 X 2); principal growth rates between adjacent frames
% R - (n_frames x 2); each row contains the eigenvalues of the (2 X 2) covariance matrix
%        of point positions for a given frame; this provides a measure of the
%        length and width of the point cloud (of nuclei) in each frame. This informaiton was
%        initially used to measure growth rates, but it is no longer the recommended way
%        as the choice of which nuclei make up the point cloud can lead to non-intuitive
%        results (e.g., if the point cloud is wide in the direction of slow growth, but
%        short in the direction of high growth, the length rate and width rate might 
%        appear to be reversed.)
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% estimate_affine_transformation.m
% hon.m
% gron.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

T = load('/proj/og1/biology/aroeder/data/flower_6/hand_tracked/dots_only/ground_truth.txt');

use = [1,2,3,4,5,6]';  % nuclei that appear in all 7 frames
n_frames = 6;
n_pts = length(use);

X = zeros(n_frames, n_pts);
Y = zeros(n_frames, n_pts);

for i = 1: n_frames
  for j = 1:n_pts
    jj = use(j);
    k = find((T(:,1) == i) & (T(:,4) == jj));
    X(i,j) = T(k,2);
    Y(i,j) = T(k,3);
  end
end

M = zeros(n_frames-1, 2);
R = zeros(n_frames, 2);
for k = 1:n_frames
  XY1 = [X(k,:)', Y(k,:)'];
  Sigma = cov(XY1);
  [U1,S1,V1] = svd(Sigma, 0);
  R(k,:) = diag(S1);
  
  if (k < n_frames)
    XY2 = [X(k+1,:)', Y(k+1,:)'];
    A = estimate_affine_transformation(XY1, XY2);
    B = A(1:2, 1:2);
    [U,S,V] = svd(B,0);
    M(k,:) = diag(S)';
  end
end
      
figure;
t = (0:5)';
hr = 12*t; % time in hours
g1 = cumprod([1;M(:,1)]);
g2 = cumprod([1;M(:,2)]);
plot(hr,g1,'m','Linewidth',2);
hon;
plot(hr,g2,'c','Linewidth',2)

%% add curves from old method (normalized to start at 1)
%plot(hr, sqrt(R(:,1)/R(1,1)), 'r');
%plot(hr, sqrt(R(:,2)/R(1,2)), 'b');

% add best fit for the new curves
alpha = 1.024;
beta  = 1.013;
plot(hr, alpha.^hr, 'g');
plot(hr, beta.^hr, 'g');

title('Flower\_6 Growth', 'Fontsize', 18);
xlabel('elapsed time (hours)', 'Fontsize', 18);
gron
keyboard
