function [M, R] = flower_6_orion()
%
% FLOWER_6_ORION: [M, R] = flower_6_orion()
%
% Use the "Orion's Belt" in flower_6 consisting of nuclei labeled 6, 1, 2
% to provide a macroscopic sepal coordinate system and measure growth rates
% along the corresponding macroscopic axes.
%
% INPUT:
% -----
%
% OUTPUT:
% ------
% M - interframe growth rates
% R - within frame estimate of length and width of point cloud
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% estimate_affine_transformation.m
% hon.m
% gron.m
% imwarp.m 
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

T = load('/proj/og1/biology/aroeder/data/flower_6/hand_tracked/dots_only/ground_truth.txt');

ibase = '/proj/og1/biology/aroeder/data/flower_6/hand_tracked/dots_overlaid/6_';
isuff = '.jpg';

orion = [6, 1, 2]'; % ordered set of labels comprinsing the belt

use = [1,2,3,4,5,6]';  % nuclei that appear in first 6 frames
n_frames = 6; % Just use first six frames, since these have all 6 nuclei present
n_pts = length(use);

X = zeros(n_frames, n_pts);
Y = zeros(n_frames, n_pts);

for i = 1: n_frames
  for j = 1:n_pts
    jj = use(j);
    k = find((T(:,1) == i) & (T(:,4) == jj));
    X(i,j) = T(k,2);
    Y(i,j) = T(k,3);
  end
end

mu_x = mean(X')';
mu_y = mean(Y')';
mu = [mu_x, mu_y];

% Determine coordinate system axes in each frame
L = 50;
C = [1, 0, 0; 0, 1, 0; 0, 0, 1; 0, 1, 1; 1, 0, 1; 0.25, 0.25, 0.25];
figure;
ab_old = zeros(n_pts,2);
BB = zeros(n_frames, 4);
for i = 1:n_frames
  x = X(i, orion);
  y = Y(i, orion);
  mu_xo = mean(x');
  mu_yo = mean(y');
  P = [x-mu_xo; y-mu_yo];
  [U,S,V] = svd(P,0);
  if (V(3,1) < 0)
    U(:,1) = -U(:,1);
  end
  if (U(:,2)'*[-U(2,1); U(1,1)] < 0)
    U(:,2) = -U(:,2);
  end
  
%  figure;
%  plot(X(i,:), Y(i,:), 'b+');
%  hon; gron; axis('ij');axis([200, 600, 300, 700]);axis('square');
%  plot(mu_x(i), mu_y(i), 'ko', 'Markersize', 12, 'Linewidth', 2);
%  plot([mu_x(i), mu_x(i)+L*U(1,1)], [mu_y(i), mu_y(i)+L*U(2,1)], 'r', 'Linewidth', 2);
%  plot([mu_x(i), mu_x(i)+L*U(1,2)], [mu_y(i), mu_y(i)+L*U(2,2)], 'g', 'Linewidth', 2);

  ab = (U' * [X(i,:)-mu_x(i); Y(i,:) - mu_y(i)])';
  % Record bounding box
  BB(i,1) = min(ab(:,1));
  BB(i,2) = max(ab(:,1));
  BB(i,3) = min(ab(:,2));
  BB(i,4) = max(ab(:,2));
  plot(0,0, 'ko', 'Markersize', 8, 'Linewidth', 2);hon;gron;
  plot(ab(:,1), ab(:,2), '+', 'Markersize', 8, 'Linewidth', 2, 'Color', C(i,:));
  for j = 1:n_pts
    plot([ab_old(j,1), ab(j,1)], [ab_old(j,2), ab(j,2)], 'Color', [0.3, 0.7, 0.3], 'Linewidth', 2);
  end
  plot_rect(BB(i,:), C(i,:), 1);
  ab_old = ab;
  
%  % Generate warped color images in orion coordinate system
%  I = imread([ibase, int2str(i), isuff]);
%  cntr = [792; 847]/2;
%  W = [U', cntr; 0, 0, 1];
%  v = W * [-mu_x(i); -mu_y(i); 0];
%  W(1:2,3) = W(1:2,3) + v(1:2);
%  J = zeros(792, 847, 3);
%  for k = 1:3
%    D = imwarp(double(squeeze(I(:,:,k))), W, 792, 847);
%    J(:,:,k) = D;
%  end
%  keyboard
end
axis('ij');
axis([-200, 150, -150, 200]); axis('square');
  

M = zeros(n_frames-1, 2);
G = zeros(n_frames-1, 2);
R = zeros(n_frames, 2);
Ak = eye(3,3);
for k = 1:n_frames
  XY1 = [X(k,:)', Y(k,:)'];
  Sigma = cov(XY1);
  [U1,S1,V1] = svd(Sigma, 0);
  R(k,:) = diag(S1);
  
  if (k < n_frames)
    XY2 = [X(k+1,:)', Y(k+1,:)'];
    A = estimate_affine_transformation(XY1, XY2);
    B = A(1:2, 1:2);
    [U,S,V] = svd(B,0);
    M(k,:) = diag(S)';
    
    Ak = [A; 0, 0, 1] * Ak;
    [U,S,V] = svd(Ak(1:2,1:2),0);
    G(k,:) = diag(S)';
    
  end
end
      
figure;
t = (0:5)';
hr = 12*t; % time in hours
g1 = cumprod([1;M(:,1)]);
g2 = cumprod([1;M(:,2)]);
plot(hr,g1,'m','Linewidth',2);
hon;
plot(hr,g2,'c','Linewidth',2)

%% add curves from old method (normalized to start at 1)
%plot(hr, sqrt(R(:,1)/R(1,1)), 'r');
%plot(hr, sqrt(R(:,2)/R(1,2)), 'b');

% add best fit for the new curves
alpha = 1.024;
beta  = 1.013;
plot(hr, alpha.^hr, 'g');
plot(hr, beta.^hr, 'g');

% Add bounding box growth
gx = BB(:,2)-BB(:,1);
gy = BB(:,4)-BB(:,3);
plot(hr, gx/gx(1), 'Color', 'b', 'Linewidth', 2);
plot(hr, gy/gy(1), 'Color', 'r', 'Linewidth', 2);

title('Flower\_6 Growth', 'Fontsize', 18);
xlabel('elapsed time (hours)', 'Fontsize', 18);
gron
keyboard
