function [Out1, Out2, Out3] = ggradient(I, smooth, polar_flag);
%
% GGRADIENT: [Out1, Out2, Out3] = ggradient(I, smooth, polar_flag);
%
% Compute the gradient of an image I.
%
% INPUT:
% -----
% I      - image
% smooth - non-negative integer indicating number of smoothing passes with 3-tap
%            binomial filter to apply to I. If smooth = 0 then we just compute
%            gradient on supplied image. The use of smooth is equivalent to
%            a binomial filter with n_taps = 2*smooth+1, so smooth=1, is equivalent
%            to a 3-tap binomial filter.
% polar_flag - 0: give output as Ix and Iy, 1: give output as Mag, Theta
%
% OUTPUT:
% ------
% if (polar_flag == 0)
%   Out1 = Ix
%   Out2 = Iy
% else
%   Out1 = Mag  (magnitude of the gradient)
%   Out2 = Theta (direction of the gradient computes as atan2(Iy, Ix))
% end
%
% Out3 = NaN mask; identifies cases where we have atan2(0,0).
%
% HISTORY:
% -------
% 2006/06/27 (MCB) - Added NaN mask for atan2(0,0) case
% 2004/02/15 (MCB) - Changed name from gradient.m to ggradient.m to avoid name 
%   collisions.
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% imsmooth.m
% set_image_borders.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

if (nargin < 3)
  polar_flag = 0;
end

ddx = [-1 0 1]/2;
ddy = ddx';
Is = imsmooth(I, smooth);

% Total support of combined derivative and smoothing filter is 2*(smooth+1)+1, which
% could leave bad border values of width smooth+1 around the image; clear these away
% In most cases, this is actually more conservative than necessary.
b = smooth+1; 
Ix = set_image_borders(conv2(1, fliplr(ddx), Is, 'same'), b, b, 0);   
Iy = set_image_borders(conv2(flipud(ddy), 1, Is, 'same'), b, b, 0);  
if (polar_flag == 0)
  Out1 = Ix;
  Out2 = Iy;
else
  Out1 = sqrt(Ix.^2 + Iy.^2);
  Out2 = atan2(Iy, Ix);
end

if (nargout > 2)
  EPS = 1e-07;
  Out3 = ( (abs(Ix) < EPS) & (abs(Iy) < EPS) );
end
