function z = gsmooth_1d(y, smooth, bound_flag);
%
% GSMOOTH_1D: z = gsmooth_1d(y, smooth, bound_flag);
%
% Applying gaussian smoothing (approximated with binomial
% coefficient filters) to a 1d profile y. The value of
% smooth indicates the number of times the 3-tap filter [ 1 2 1 ]/4
% is applied to the profile. This is equivalent to using
% a binomial coefficient filter with 2*smooth+1 taps.
% The user may specify how to handle the profile endpoints.
% 
%
% INPUT:
% -----
% y        - column vector; values to be smoothed
% smooth   - nonnegative int; number of passes with 3 tap binomial coeff filter
% bound_flag - method of handling ends of f segments. Options are as follows:
%
%            'zero' - pad with zeros
%            'hold' - repeat first and last values of F
%            'wrap' - circularly wrap
%            'xtrp' - extrapolate values (using spline)
%
% OUTPUT:
% ------
% z       - a gaussian smoothed version of y (same size)
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

% Quick exit if no smoothing requested
if (smooth == 0)
  z = y;
  return;
end

% Determine boundary method or set to DEFAULT
if (nargin < 3)
  bnd = 'zero';
else
  bnd = bound_flag(1:4);
end

if (strcmp(bnd,'zero'))
  method = 0;
elseif (strcmp(bnd,'hold'))
  method = 1;
elseif (strcmp(bnd,'wrap'))
  method = 2;
elseif (strcmp(bnd,'xtrp'))
  method = 3;
else
  error('Unsupported boundary flag');
end

n = length(y);
w = 2*smooth+1;
ww = smooth;

% Pad y using the desired boundary method
if (method==0)
  yy = [zeros(ww,1); y; zeros(ww,1)];
elseif (method == 1)
  yy = [y(1) * ones(ww,1); y; y(n) * ones(ww,1)];
elseif (method == 2)
  yy = [y(n-ww+1:n); y; y(1:ww)];
else
  xold = 1:n;
  xnew = -ww+1:n+ww;
  yy = interp1(xold,y,xnew,'spline');
end

% Apply desired number of iterations with basic kernel
g0 = [1; 2; 1]/4;
zz = yy;
for iter = 1:smooth
  zz = conv2(g0, 1, zz, 'same');
end
z = zz(ww+1:ww+n, 1);


