function [L, nobj] = icluster(I, dthr)
%
% ICLUSTER: [L, nobj] = icluster(I, dthr);
%
% Apply the sequential labeling algorithm to a binary image I. Any pixels
% within dthr of each other are assigned to the same
% cluster. This rule is applied transitively. Distance is measured using
% the infinity norm (max(|dx|, |dy|)).
%
% INPUT:
% -----
% I    - binary image
% dthr - (2 x 1) vector; distance threshold for grouping
%          dthr(1) is the distance threshold on row differences (vertical)
%          dthr(2) is the distance threshold on col differences (horizontal)
% 
% OUTPUT:
% ------
% L    - integer-valued image of cluster labels (same size as I)
% nobj - number of objects (clusters) found
%
% NOTE:
% ----
% M is an important internal variable used in this function.
% M(l) represents the "object number" associated with label l.
%
% HISTORY:
% -------
% 2004/03/01 (MCB) - Changed dthr to be a 2x1 vector to allow different aggressiveness
%   for grouping horizontally and vertically.
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

if (length(dthr) == 1)
  dthr = [dthr; dthr];
end

[nr, nc] = size(I);
L = zeros(nr, nc);

% Count number of non-zero elements in I as an upper bound on number of objects
n = sum(sum(I));
M = zeros(n,1);

new_label = 1;
new_object = 1;
num_objects = 0;

for i=1:nr
  for j = 1:nc
    if (I(i,j) ~= 0)
      if (L(i,j) == 0)
	L(i,j) = new_label;
	M(new_label) = new_object;
	new_label = new_label+1;
	new_object = new_object+1;
	num_objects = num_objects+1;
      end
      
      i0 = i; % relies on top to bottom scanning strategy
      i1 = min(nr, i+dthr(1));
      j0 = max(1, j-dthr(2));
      j1 = min(nc, j+dthr(2));
      for ii=i0:i1
	for jj = j0:j1
	  if (I(ii,jj) ~= 0) 
	    % Pixel at (i,j) and (ii,jj) belong in the same cluster
	    if (L(ii,jj) == 0)
	      % Point at (ii, jj) has not been labeled yet
	      L(ii,jj) = L(i,j);
	    elseif (L(ii,jj) ~= L(i,j))
	      % Make a note that label L(i,j) and L(ii,jj) are equivalent
	      if (M(L(i,j)) ~= M(L(ii,jj)))
		% if equivalence is not already known
		num_objects = num_objects - 1;
		lo_obj = min(M(L(i,j)), M(L(ii,jj)));
		hi_obj = max(M(L(i,j)), M(L(ii,jj)));		
		% Change all hi_obj's to lo_obj's
		ind = find(M == hi_obj);
		M(ind) = lo_obj;
	      end
	    % else L(i,j) already equals L(ii,jj); don't have to do anything
	    end
	  end
	end
      end
    end
  end
end

num_labels = new_label - 1;
    
%---------------------------
% SECOND PASS - compression
%---------------------------
    
% The labeling procedure above potentially leaves holes in the
% object numbers; here we do a second pass to compress the
% numbers into consecutive integers.
    
m_unique = unique(M(1:num_labels));
nobj = length(m_unique);

F = zeros(num_labels,1);
for l = 1:num_labels
  F(l) = find(m_unique == M(l)); % where does this object number
end

for i = 1:nr
  for j = 1:nc
    if (L(i,j) ~= 0)
      L(i,j) = F(L(i,j));
    end
  end
end

    
	      