function A = icluster_moments(L, nobj, EPS)
%
% ICLUSTER_MOMENTS: A = icluster_moments(L, nobj, EPS);
%
% Given a label image L, determine the moments (0th, 1st, 2nd) for each cluster.
%
% INPUT:
% -----
% L    - label image (e.g., from icluster)
% nobj - number of objects (number of labels in L)
% EPS  - standard deviation assigned to singleton clusters
%
% OUTPUT:
% ------
% A    - (nobj x 6) matrix. 1st col = # of elements, 2nd col = x_mean, 3rd col = y_mean
%           4th col = Sxx, 5th col = Sxy, 6th col = Syy
%
% NOTE:
% ----
% Singleton clusters (clusters with only one element) are assigned a default 
% circular covariance of EPS^2 * I
%
% HISTORY:
% -------
% 2004/03/01 (MCB) - Added a check for empty clusters
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

if (nargin < 3)
  EPS = 0.0001; 
end

A = zeros(nobj, 6);
for l = 1:nobj
  [y,x] = find(L == l);
  n = length(y);
  if (n == 0)
    A(l,:) = [0, 0, 0, EPS^2, 0, EPS^2];
  elseif (n == 1)
    mux = mean(x);
    muy = mean(y);
    A(l,:) = [n, mux, muy, EPS^2, 0, EPS^2];  
  else
    mux = mean(x);
    muy = mean(y);
    S = cov([x,y]);
    A(l,:) = [n, mux, muy, S(1,1), S(1,2), S(2,2)];
  end
end