function D = imwarp(S, P, dr, dc)
%
% IMWARP: D = imwarp(S, P, dr, dc);
%
% Apply a 2D projective warping (homography) P to the
% image S to produce a new image D of size (dr X dc).
% Affine transformations and others are special cases of P.
%
% INPUT:
% -----
% S - source image
% P - 3 X 3 matrix describing a 2-D projective mapping from
%       source space (u,v) to destination space (x,y). 
%
%       P * [u, v, 1]' = [xw, yw, w]';
%
% dr - desired number of rows in destination image D
% dc - desired number of cols in destination image D
%
% OUTPUT:
% ------
% D  - (dr X dc) image
%
% Note 1: Although P has 9 parameters, there are only 8 degrees of
%           freedom since multiplying P by a non-zero scalar gives the
%           same mapping.
% Note 2: P is affine iff P(3,1)=P(3,2)=0 and P(3,3) ~= 0
% Note 3: My P is the transpose of Heckbert's P so that P acts
%           from the left on column vectors.
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

[sr, sc] = size(S);
LTAB = 256;
alpha = 16/LTAB;
WTAB = exp(-alpha * (0:LTAB-1));
D = zeros(dr, dc);

% Need adj(P) = cof(A)^T to map from destination coords back 
%   to source coords. adj(P) * P = det(P) * eye(3,3);
R = zeros(3,3);
R(1,1) = P(2,2)*P(3,3) - P(2,3)*P(3,2);
R(2,1) = P(2,3)*P(3,1) - P(2,1)*P(3,3);
R(3,1) = P(2,1)*P(3,2) - P(2,2)*P(3,1);
R(1,2) = P(1,3)*P(3,2) - P(1,2)*P(3,3);
R(2,2) = P(1,1)*P(3,3) - P(1,3)*P(3,1);
R(3,2) = P(1,2)*P(3,1) - P(1,1)*P(3,2);
R(1,3) = P(1,2)*P(2,3) - P(1,3)*P(2,2);
R(2,3) = P(1,3)*P(2,1) - P(1,1)*P(2,3);
R(3,3) = P(1,1)*P(2,2) - P(1,2)*P(2,1);

for x = 1:dc
  for y = 1:dr
    % Compute source space ellipse center
    s  = (R(3,1)*x + R(3,2)*y + R(3,3));
    us = (R(1,1)*x + R(1,2)*y + R(1,3));
    vs = (R(2,1)*x + R(2,2)*y + R(2,3));
    U0 = us/s;
    V0 = vs/s;
    
    % Compute (Ux,Vx) and (Uy,Vy) (Jacobian of the mapping)
    Ux = (R(1,1) - U0 * R(3,1))/s;
    Vx = (R(2,1) - V0 * R(3,1))/s;
    Uy = (R(1,2) - U0 * R(3,2))/s;
    Vy = (R(2,2) - V0 * R(3,2))/s;
    
    A = Vx*Vx + Vy*Vy+1;
    B = -2 * (Ux*Vx+Uy*Vy);
    C = Ux*Ux + Uy*Uy + 1;
%    F = (Ux*Vy-Uy*Vx)^2;
    F = A*C - B*B/4;
    
    % Ellipse is AU^2 + BUV + CV^2 = F where U = u-U0 and V=v-V0
    % Find bounding box around ellipse [u1,u2] X [v1,v2]
    delta = A*C - B*B/4;
    uu = C*F/delta;
    vv = A*F/delta;    
    u1 = max(1, floor(U0 - uu));
    u2 = min(sc, ceil(U0 + uu));
    v1 = max(1, floor(V0 - vv));
    v2 = min(sr, ceil(V0 + vv));

    % Scale A,B,C, and F so that F = WTAB length
    scale = (LTAB-1)/F;
    A = A * scale;
    B = B * scale;
    C = C * scale;    
    F = F * scale;

    NUM = 0;
    DEN = 0;
    DDQ = 2*A;
    U = u1-U0;
    for v = v1:v2
      V = v-V0;
      DQ = A*(2*U+1) + B*V;
      Q = (C*V+B*U)*V + A*U*U;
      
      for u=u1:u2
	if (Q <= F)
	  if (Q < 0)
	    % Possible -epsilon error
	    Q = 0;
	  end
	  WEIGHT = WTAB(1+floor(Q));
	  NUM = NUM+WEIGHT*S(v,u);
	  DEN = DEN+WEIGHT;
	end
	Q = Q+DQ;
	DQ = DQ+DDQ;
      end
    end
    if (DEN == 0)
      ul = floor(U0);
      vl = floor(V0);
      if ((ul >= 1) & (ul < sc) & (vl >= 1) & (vl < sr))
	% bilinear fit
        D(y,x) = (ul+1-U0)*(vl+1-V0)*S(vl,ul)+(ul+1-U0)*(V0-vl)*S(vl+1,ul)+(U0-ul)*(vl+1-V0)*S(vl,ul+1)+(U0-ul)*(V0-vl)*S(vl+1,ul+1);
      else
        D(y,x) = 0;
      end
    else
      D(y,x) = NUM/DEN;
    end
  end
end
