function S = intersect_plane_triangle(P, T)
%
% INTERSECT_PLANE_TRIANGLE: S = intersect_plane_triangle(P, T)
%
% Compute the intersection between a plane and a triangle; the result may be empty,
% a signle point, a line segment, or a whole triangle.
%
% INPUT:
% -----
% P  - (4 x 1) vector describing plane in homogeneous coordinates
%              P(1:3) defines unit normal; P(4) is (signed) distance 
%              from plane to the origin along the unit normal direction.
% T  - (3 x 3) matrix of vertices; each ROW is one vertex
%
% OUTPUT:
% ------
% S  - (n x 3) matrix of vertices in intersection
%      NOTE:   if n=0, intersection is empty
%              if n=1, intersection is a single point
%              if n=2, intersection is a line segment
%              if n=3, intersection is whole triangle
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------
EPS = 1e-12;

% Construct new coordinate system
L = norm(P(1:3));
if (L == 0)
  fprintf(1, 'ERROR (intersect_plane_triangle): normal to plane must have non-zero length\n');
  S = [];
  return
end

n_hat = P(1:3,1)/L;
d = P(4,1)/L;
if (n_hat(1) ~= 0)
  v1 = [-n_hat(2); n_hat(1); 0];
elseif (n_hat(2) ~= 0)
  v1 = [0; -n_hat(3); n_hat(2)];
elseif (n_hat(3) ~= 0)
  v1 = [-n_hat(3); 0; n_hat(1)];
end
v1_hat = v1/norm(v1);
v2_hat = cross(n_hat, v1_hat);
pTw = zeros(4,4);
pTw(4,4) = 1;
pTw(1:3,1:3) = [v1_hat'; v2_hat'; n_hat'];
pTw(3,4) = d;

wTp = inv(pTw);

% Convert triangle vertices to new coordinate system
Tnew = (pTw * [T'; ones(1,3)])'; % rows of Tnew have homogeneous coords

% Test 0: straddle test - does triangle straddle z=0?
z = Tnew(1:3,3);
zmin = min(z);
zmax = max(z);

if ((zmin > 0) | (zmax < 0))
  S = [];
  return;
end

% Determine how many points in an abs(z) < EPS band
zabs = abs(z);
ind = find(zabs < EPS);  % indices of points in plane
oth = find(zabs >= EPS); % indices of points not in plane
cnt = length(ind);

% Test 1: check for single point intersection
if (cnt == 1)
  if (sign(z(oth(1))) == sign(z(oth(2))))
    % intersection is one vertex
    S = T(ind,:);
    return;
  end
end

% Test 2: check for two points in EPS band
if (cnt == 2)
  S = T(ind,:);
  return
end

% Test 3: triangle in plane test
if (cnt == 3)
  % triangle lies in plane
  S = T;
  return;
end

% Test 5: if reach here, intersection is a line segment
S = [];
for k = 1:3
  l = k+1;
  if (l == 4)
    l = 1;
  end

  if ( ((z(k) <= EPS) & (z(l) >= -EPS)) | ((z(k) >= -EPS) & (z(l) <= EPS)) )
    % Edge (k, l) has an intersection
    if (z(k) == z(l))
      keyboard
    end
    t = z(k)/(z(k)-z(l));
    Pnew = (1-t) * Tnew(k,1:3) + t * Tnew(l,1:3);
    P = (wTp * [Pnew'; 1])';
    S = [S; P(1,1:3)];
  end
end


