function I = lineage_label_info(t, tgt, T, LL, abs_flag)
%
% LINEAGE_LABEL_INFO: I = lineage_label_info(t, tgt, T, LL, abs_flag)
%
% Determine relationship between a target lineage label (such as 4.1.2) and
% a set of lineage labels held in the cell array LL, e.g., find indices
% of points in LL that contain any direct child of the target label (such as 4.1.2.1).
%
% INPUT:
% -----
% t        - scalar; time of interest
% tgt      - string with target label
% T        - (n x 1) matrix of timestamps
% LL       - (n x 1) cell array of lineage labels 
% abs_flag - 0 = do not use absolute value on lineage labels, 1 = use absolute value 
%
% OUTPUT:
% ------
% I   - (5 x 1) cell array;
%
%         I{1,1} - indices of any ancestors of tgt
%         I{2,1} - indices of any direct parent of tgt
%         I{3,1} - indices of tgt
%         I{4,1} - indices of any direct child of tgt
%         I{5,1} - indices of any descendents of tgt
%
% NOTE: There is no mechanism for siblings or cousins at this point.
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% lineage_label_to_vector.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

if (nargin < 5)
  abs_flag = 0;
end

% Initialize output array
I = cell(5,1);
for k=1:5
  I{k,1} = [];
end


h0 = lineage_label_to_vector(tgt);
g0 = length(h0);
if (abs_flag == 1)
  h0 = abs(h0);
end

ind = find(T == t);
if (~isempty(ind))
  for i = 1:length(ind)
    l = LL{ind(i),1}; 

    % Break this label into pieces
    h1 = lineage_label_to_vector(l);
    g1 = length(h1);
    if (abs_flag == 1)
      h1 = abs(h1);
    end
    
    % What is relation of this label to tgt?
    if (g1 == g0)
      % Check for exact match
      if (max(abs(h1-h0)) == 0)
	I{3,1} = [I{3,1}; ind(i)];
      end
    elseif (g1 == (g0+1))
      % Check for child relation
      if (max(abs(h1(1:g0)-h0)) == 0)
	I{4,1} = [I{4,1}; ind(i)];
      end
    elseif (g1 > (g0+1))
      % Check for descendant relation
      if (max(abs(h1(1:g0)-h0)) == 0)
	I{5,1} = [I{5,1}; ind(i)];
      end
    elseif (g1 == (g0-1))
      % Check for parent relation
      if (max(abs(h1-h0(1:g1))) == 0)
	I{2,1} = [I{2,1}; ind(i)];
      end
    elseif (g1 < (g0-1))
      % Check for ancestor relation
      if (max(abs(h1-h0(1:g1))) == 0)
	I{1,1} = [I{1,1}; ind(i)];
      end
    else
      fprintf(1, 'ERROR (lineage_label_info): should nto reach this ELSE\n');
      return
    end
  end
end
