function [Ce, Cb, nobj] = link_segments(E, B, dthr)
% LINK_SEGMENTS: [Ce, Cb, nobj] = link_segments(E, B, dthr);
%
% Link a set of unordered segments into sets of contours. 
%
% INPUT:
% -----
% E  - (N x 4) matrix; each row contains one segment  [x1, y1, x2, y2]
% B -  (6F x 4) matrix; each row contains one segment [x1, y1, x2, y2]
%         groups of 6 rows form a "butterfly" across a single offset pixel.
% dthr - [optional] distance threshold between segment endpoints for merging
%
% OUTPUT:
% ------
% Ce  - (N x 1) integer label vector assigning each segment to a contour
% Cb  - (F x 1) integer label vector assigning each butterfly to a contour
% nobj - number of objects (contours) found
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

if (nargin < 3)
  dthr = 1;
end

[er, ec] = size(E);
if (ec ~= 4)
  fprintf(1, 'ERROR (link_segments): E must have 4 columns\n');
  Ce = [];
  Cb = [];
  nobj = 0;
  return;
end

[br, bc] = size(B);
if (br > 0)
  if (bc ~=4)
    fprintf(1, 'ERROR (link_segments): B must have 4 columns\n');
    Ce = [];
    Cb = [];    
    nobj = 0;
    return;
  end
end
nb = br/6;
if (nb ~= floor(nb))
  fprintf(1, 'ERROR (link_segments): B must have multiple of 6 rows\n');
  Ce = [];
  Cb = [];    
  nobj = 0;
  return;
end

% Determine maximum dimensions
bnds = max([E; B]);
nr = 1 + ceil(max(bnds(2), bnds(4)));
nc = 1 + ceil(max(bnds(1), bnds(3)));

I = zeros(nr, nc); % Offset lattice

% Into which pixel does each segment fall?
for e = 1:er
  ii = 1+ floor((E(e,2) + E(e,4))/2);
  jj = 1+ floor((E(e,1) + E(e,3))/2);  
  I(ii,jj) = e;
end

% Into which pixel does each butterfly fall?
for k = 1:nb
  b = (k-1)*6+1;
  ii = 1+ floor((B(b,2) + B(b,4))/2);
  jj = 1+ floor((B(b,1) + B(b,3))/2);  
  I(ii,jj) = -k;  % give butterflies negative values in I to distinguish from segments
end

%---------------------------------------------------------------------------------------------
% Apply specialized version of connected components - very much like icluster.m
L = zeros(nr, nc);

% Number of segments+butterflies is an upper bound on number of objects
M = zeros(er+nb, 1);

new_label = 1;
new_object = 1;
num_objects = 0;

for i=1:nr
  for j = 1:nc
    v1 = I(i,j);
    if (v1 ~= 0)
      if (L(i,j) == 0)
	L(i,j) = new_label;
	M(new_label) = new_object;
	new_label = new_label+1;
	new_object = new_object+1;
	num_objects = num_objects+1;
      end
      
      if (v1 > 0)
        epts1 = [E(v1,1:2); E(v1,3:4)];
      else
	epts1 = [B(-6*v1-5,1:2); B(-6*v1-5,3:4); B(-6*v1-3,1:2); B(-6*v1-3,3:4)];
      end
      n1 = size(epts1,1);
      
      i0 = i; % relies on top to bottom scanning strategy
      i1 = min(nr, i+1);
      j0 = max(1, j-1);
      j1 = min(nc, j+1);
      for ii=i0:i1
	for jj = j0:j1
	  v2 = I(ii,jj);
	  if (v2 ~= 0) 
            if (v2 > 0)
              epts2 = [E(v2,1:2); E(v2,3:4)];
            else
	      epts2 = [B(-6*v2-5,1:2); B(-6*v2-5,3:4); B(-6*v2-3,1:2); B(-6*v2-3,3:4)];
            end
	    n2 = size(epts2,1);

	    % Element at (i,j) and (ii,jj) may belong in the same cluster
	    %   Need to do endpoint test	    
	    d = zeros(n1*n2,1);
	    kk = 0;
	    for t1=1:n1
	      for t2 =1:n2
		kk = kk+1;
		d(kk,1) = sqrt( sum( (epts1(t1,:)-epts2(t2,:)).^2 ) );
	      end
	    end
            dmin = min(d);
	    if (dmin < dthr)
	      if (L(ii,jj) == 0)
		% Point at (ii, jj) has not been labeled yet
		L(ii,jj) = L(i,j);
	      elseif (L(ii,jj) ~= L(i,j))
		% Make a note that label L(i,j) and L(ii,jj) are equivalent
		if (M(L(i,j)) ~= M(L(ii,jj)))
		  % if equivalence is not already known
		  num_objects = num_objects - 1;
		  lo_obj = min(M(L(i,j)), M(L(ii,jj)));
		  hi_obj = max(M(L(i,j)), M(L(ii,jj)));		
		  % Change all hi_obj's to lo_obj's
		  ind = find(M == hi_obj);
		  M(ind) = lo_obj;
		end
	      % else L(i,j) already equals L(ii,jj); don't have to do anything
	      end
	    end
	  end
	end
      end
    end
  end
end

num_labels = new_label - 1;
    
%---------------------------
% SECOND PASS - compression
%---------------------------
    
% The labeling procedure above potentially leaves holes in the
% object numbers; here we do a second pass to compress the
% numbers into consecutive integers.
    
m_unique = unique(M(1:num_labels));
nobj = length(m_unique);

F = zeros(num_labels,1);
for l = 1:num_labels
  F(l) = find(m_unique == M(l)); % where does this object number
end

for i = 1:nr
  for j = 1:nc
    if (L(i,j) ~= 0)
      L(i,j) = F(L(i,j));
    end
  end
end

%---------------------------------------------------------------------------------------------
keyboard

% Convert connected components result back to contour labels
Ce = zeros(er,1);
Cb = zeros(nb,1);
for i = 1:nr
  for j = 1:nc
    if (L(i,j) ~= 0)
      v = I(i,j);
      if (v > 0)
        Ce(v,1) = L(i,j);
      elseif (v < 0)
        Cb(-v,1) = L(i,j);	
      end
    end
  end
end


