function [L, XY] = link_vertices(f)
%
% LINK_VERTICES: [L, XY] = link_vertices(f);
%
% Looks like this function loads a membrane image and a set of points that represent
% vertices (e.g., acquired by hand clicking). The function then tries to establish
% which links should exist between the vertices based on the membrane image.
%
% INPUT:
% -----
% f - frame number
%
% OUTPUT:
% ------
% L  - (n_links X 2); each row of L is a "link" containing two numbers specifying indices of
%        vertices in XY that should be connected by a link.
% XY - (n_vertices X 2); x and y coordinates of vertices (junctions between membrane segments)
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% construct_fname.m
% distmat.m
% oriented_box_integral.m
% salient_points.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------
ibase = '/proj/og1/biology/mheisler/data/for_eric/membranes/dye';
isuff = '.tif';
vbase = '/proj/og1/biology/mheisler/ground_truth/vertices_';
vsuff = '.txt';
fwidth = 3;

w = 7;
maxlink = 100;
a_thr = cos(pi/12);
EPS = 1e-07;

iname = construct_fname(ibase, isuff, f, fwidth);
vname = construct_fname(vbase, vsuff, f, fwidth);
I = sqrt(double(imread(iname)));
XY = load(vname);

fig(sqrt(I)); hon; plot(XY(:,1), XY(:,2), 'bo');

sf = 2;
W = 15;
H = 3;
R = 5 + (W-1)/2;
ovec = (0:5:405)';
cv = cos(ovec*pi/180);
sv = sin(ovec*pi/180);

V = zeros(length(ovec),1);

Cset = cell(1,1);
[nr, nc] = size(I);
nv = size(XY,1);
fprintf(1, 'nv = %d\n', nv);
D = distmat(XY, XY);

T = cell(nv,1);
N = zeros(nv,1);
for v1 = 1:nv
%  fprintf(1, '%d\n', v1);
  x0 = XY(v1,1);
  y0 = XY(v1,2);
  for k = 1:length(ovec)
    o = ovec(k,1);
    [s,n] = oriented_box_integral(I,x0+R*cv(k,1),y0+R*sv(k,1), W, H, o);
    V(k,1)=s/n;
  end
  V = V-min(V);
%  plot(x0+V.*cv*sf, y0+V.*sv*sf, 'm');
  Vset = cell(1,1); Vset{1,1} = V;
  indset = salient_points(Vset, 9, [-Inf; -Inf]);
  ii = indset{1,1};
%  for l = 1:length(ii)
%    plot(x0+V(ii(l),1).*cv(ii(l),1)*sf, y0+V(ii(l),1).*sv(ii(l),1)*sf, 'c.');
%  end

  if (~isempty(ii))
    T{v1,1} = ii; % Likely exit indices from this junction
  else
    T{v1,1} = [];
  end
  N(v1,1) = length(ii);

%  for v2 = v1+1:nv
%    if (D(v1, v2) < maxlink)
%      Cset{1,1} = [XY(v1,:); XY(v2,:)]; % single segment from v1 to v2
%      B = cset_to_bitmap(nr, nc, Cset);
%      Bb = conv2(ones(w,1), ones(w,1), B, 'same');
%      A(v1, v2) = sum(sum(I .* (Bb > 0)))/ sum(sum(Bb > 0));
%      A(v2, v1) = A(v1,v2);
%    end
%  end
end

ntot = sum(N);
F = zeros(ntot, ntot);
VV = zeros(ntot, 1);

k1 = 0;
for v1 = 1:nv
  i1 = T{v1,1};
  for l1 = 1:length(i1)
    k1 = k1+1;
    VV(k1,1) = v1;
    dx1 = cv(i1(l1));
    dy1 = sv(i1(l1));

    k2 = 0;
    for v2 = 1:nv
      i2 = T{v2,1};      
      for l2 = 1:length(i2)
	k2 = k2+1;
	du = XY(v2,1) - XY(v1,1);
	dv = XY(v2,2) - XY(v1,2);
	r = sqrt(du*du + dv*dv);
	if (r > EPS)
  	  du = du/r;
	  dv = dv/r;
	

	  dx2 = cv(i2(l2));
	  dy2 = sv(i2(l2));

	  s1 = du*dx1 + dv*dy1;
	  s2 = -du*dx2 - dv*dy2;
	  F(k1, k2) = (s1+s2)/2;
	end
      end
    end
  end
end

% Limit distance of interactions
for k1 = 1:ntot
  for k2 = 1:ntot
    if (D(VV(k1), VV(k2)) > maxlink)
      F(k1,k2) = -Inf;
    end
  end
end

% For each (node, exit_angle) find best match
R = zeros(ntot,1);
S = zeros(ntot,1);
for k1 = 1:ntot
  [fmax, imax] = max(F(k1,:));
  R(k1,1) = fmax(1);
  S(k1,1) = imax(1);
end

% Apply symmetry test
keep = ones(ntot, 1);
for k1 = 1:ntot
  if (S(S(k1)) ~= k1)
    keep(k1,1) = 0;
  end
end

n_links = sum((R > a_thr) & (keep == 1));
L = zeros(n_links, 2);
l = 0;
for k1 = 1:ntot
  if (R(k1) > a_thr)
    ii = VV(k1);
    jj = VV(S(k1));
%    plot([XY(ii,1), XY(jj,1)], [XY(ii,2), XY(jj,2)], 'g', 'linewidth', 1);
    if (keep(k1) == 1)
      l = l+1;
      L(l,:) = [ii, jj];
      plot([XY(ii,1), XY(jj,1)], [XY(ii,2), XY(jj,2)], 'r', 'linewidth', 2);      
    end
  end
end
keyboard
