function L = linkage_from_voxelized(V, C)
%
% LINKAGE_FROM_VOXELIZED: L = linkage_from_voxelized(V, C)
%
% Given a voxelization of a set of cell compartments, construct a linkage structure
% (cell array) showing the affinity of object id (oid) i in level k with oid j in level 
% k+1, where the affinity is based on the intersection area normalzied by the union area.
%
% INPUT:
% -----
% V - (nx X ny X nz) voxelization; each voxel contains an object id (or 0=background)
% C - (n_colors X 3); color matrix
%
% OUTPUT:
% ------
% L - (nz X 1) cell array. L{k,1} = matrix showing linkage between level k-1 and k
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% hon.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

[nx, ny, nz] = size(V);
n_cells = max(max(max(V))); % Number of (biological) cells or object ids
L = cell(nz,1);

old_mask = zeros(nx, ny);
for k=1:nz
  mask = squeeze(V(:,:,k)); % slice through the voxelization at z=k
  
  if (max(max(mask)) > 0)
    AB    = zeros(n_cells, n_cells);
    ABbar = zeros(n_cells, n_cells);  
    AbarB = zeros(n_cells, n_cells);  
    for m=1:n_cells
      for n = 1:n_cells
        AB(m,n)    = sum(sum((old_mask == m) & (mask == n)));
        ABbar(m,n) = sum(sum((old_mask == m) & (mask ~= n)));
        AbarB(m,n) = sum(sum((old_mask ~= m) & (mask == n)));      
      end
    end
    L{k,1} = AB./(AB + ABbar + AbarB + 0.001); % NOTE: L{k,1} = link between level k-1 and k
  else
    L{k,1} = zeros(n_cells, n_cells);
  end
  old_mask = mask;
end

keyboard
figure; 
hon;

% DRAW THE NODES
rho = zeros(nz, n_cells);
for k = 1:nz
  mask = squeeze(V(:,:,k)); % slice through the voxelization at z=k
  for m = 1:n_cells
    rho(k,m) = sum(sum(mask == m)); % area occupied by label m at level k.
  end
end

for k = 1:nz
  mind = find(rho(k,:) > 0); % labels at level k with non-zero area
  for mm = 1:length(mind)
    m = mind(mm);
    plot(k, mm, 'o', 'Color', C(m,:), 'Markersize', 12, 'Linewidth', 2);
    text(k-0.3, mm-0.2, int2str(m));	  
  end
end

% DRAW THE LINKS
for k=1:nz
  R = L{k,1};
  if (max(max(R)) > 0) % there are some non-null links between layer k-1 and k
    for m=1:n_cells
      for n=1:n_cells
        w = floor(sqrt(R(m,n))*10); 
        if (w > 0)
	  mind = find(rho(k-1,:) > 0);
	  mm = find(mind == m);
	  nind = find(rho(k,:) > 0);
	  nn = find(nind == n);
          if (m == n)
            plot([k-1,k], [mm,nn], 'Color', 'b', 'Linewidth',w);
	  else
  	    plot([k-1,k], [mm,nn], 'Color', 'r', 'Linewidth',w);
	  end
        end
      end
    end
  end
end
axis('ij')
