function [x, y, t, v] = local_extrema(I, w, tgts)
%
% LOCAL_EXTREMA: [x, y, t, v] = local_extrema(I, w, tgts);
%
% Find points in I that are local extrema over a window of size (w x w). Optionally,
% can search for only local maxima, only local minima, or both.
%
% INPUT:
% -----
% I    - (nr X nc) grayscale image
% w    - width of window over which a point must be an extrema
% tgts - scalar; +1 = look for maxima only, -1 = look for minima only, 0 = look for both (def) 
%
% OUTPUT:
% ------
% x   - (N x 1) vector with x-coordinates of local extrema
% y   - (N x 1) vector with y-coordinates of local extrema
% t   - (N x 1) vector with type of extrema (+1 = local max, -1 = local min)
% v   - (N X 1) vector with value of I at the extrema
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

if (nargin < 3)
  tgts = 0;
end
if (nargin < 2)
  w = 5;
end

wm = floor((w-1)/2);
wp = floor(w/2);

[nr,nc] = size(I);

% Pre-allocate
x = zeros(nr*nc,1);
y = zeros(nr*nc,1);
t = zeros(nr*nc,1);
v = zeros(nr*nc,1);

k = 0;
for i = 1:nr
  ilo = max(i-wm, 1);
  ihi = min(i+wp, nr);
  for j = 1:nc
    jlo = max(j-wm, 1);
    jhi = min(j+wp, nc);
    W = I(ilo:ihi,jlo:jhi);
    if (tgts >= 0)
      if (I(i,j) == max(W(:)))
        k = k+1;
        x(k,1) = j;
        y(k,1) = i;
        t(k,1) = 1;
        v(k,1) = I(i,j);
      end
    end
    if (tgts <= 0)
      if (I(i,j) == min(W(:)))
        k = k+1;
        x(k,1) = j;
        y(k,1) = i;
        t(k,1) = -1;
        v(k,1) = I(i,j);      
      end
    end
  end
end
 
if (k > 0)
  x = x(1:k,1);
  y = y(1:k,1);
  t = t(1:k,1);
  v = v(1:k,1);  
else
  x = [];
  y = [];
  t = [];
  v = [];  
end
