function [w, t, r, e, s, t_best, r_best, e_best] = lqs2(X, q);
%
% LQS2: [w, t, r, e, s, t_best, r_best, e_best] = lqs2(X, q). 
%
% Perform robust regression using Least Quantile of Squares followed by
%   Reweighted Least Squares to find T such that quantile(((X*T).^2),q) is minimized
%   with the additional constraint that the last element of T = -1. Note that q = 0.5
%   corresponds to the usual least median of squares algorithm.
%   R is just the vector of residuals (X*T). This procedure works probabilistically
%   so it cannot be guaranteed that the optimal answer will always be found.
%   By doing enough iterations the probability of generating the correct
%   answer can be insured to exceed some predetermined level (say 95%).
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% choose.m
% quantile.m
% phiinv.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

[n,m]   = size(X);
alpha   = 0.999;   % probability a trial submatrix of _good_ data is selected
epsilon = 0.5;     % percent of data rows that may be corrupted
beta    = m-1;     % number of data rows in each trial submatrix

qq      = floor(q * n);

% Determine number of iterations required to insure confidence level
niter = ceil(log(1-alpha) / log(1 - (1-epsilon).^beta));

% Establish initial values
e_best = inf;

% MAIN LOOP
for i = 1 : niter
  % Randomly select beta rows of X
  A = X(choose(beta,n),:);  % A is a (beta X m) matrix

  % Determine t such that A*t = 0;
  t_trial = [pinv(A(:,1:m-1)) * A(:,m); -1];   % Least squares based on submatrix

  % See how t works on _all_ the rows of X
  r_trial = X * t_trial;
  e_trial = sqrt(quantile(r_trial.^2, q));

  if (e_trial < e_best)
    e_best = e_trial;
    r_best = r_trial;
    t_best = t_trial;
  end
end

% Estimate the scale of the residuals and the reweighting weights
s = 1/phiinv(1-q/2) * e_best; % May not be accurate for small sample sizes 
  
% Added check on s to avoid divide by zero (MCB - 001227)  
zs = find(s == 0);
if (length(zs) > 0)
  s(zs) = ones(length(zs),1);
  w = (abs(r_best/s) <= 2.5);   % Weights (w = 0 for outliers, 1 for good data);
  w(zs) = s(zs);
else
  w = (abs(r_best/s) <= 2.5);   % Weights (w = 0 for outliers, 1 for good data);
end

    
%h = sort(abs(r_best));
%w = (abs(r_best) <= h(qq));

% Recompute t on good data
t = [pinv(X(w,1:m-1)) * X(w,m); -1];
r = X * t;
e = sqrt(quantile(r.^2, q));

