function B = myresize(A, M, method)
%
% MYRESIZE: B = myresize(A, M, method);
% 
% Program to resize images. A is the input image, B is the resized
% output image. M may either be a scalar or a 1 X 2 row vector. If
% M is a scalar then size(B) = M * size(A). If M is a vector, then
% size(B) = M. Method is a string specifying which interpolation
% method should be used. Available choices are 'nearest', 'bilinear',
% and 'bicubic'.
%
% NOTE: When the image A is to be reduced in size, it is first low-pass
% filtered to avoid aliasing.
%
% Adapted from the routine IMRESIZE in the Mathworks Image Processing
% Toolbox.
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% myfir1.m
% bestblk.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------
 
[ar, ac] = size(A);

if (nargin < 3) 
  kase = 'bil';
else
  kase = method(1:3);
end

if (prod(size(M)) == 1)
  bsize = floor(M * size(A)); % M is a scalar resizing factor
else
  bsize = M;                  % M is a vector specifying output size
end
br = bsize(1,1); bc = bsize(1,2);


% Determine if there is any shrinking required. If so, we
% need to low-pass filter first in order to avoid aliasing.
% Antialiasing is not done if using method = 'nearest'.
nn = 11; h = [];
if (kase(1)=='b')
  if (br < ar)
    h1 = myfir1(nn-1, br/ar);
  else 
    h1 = 1;
  end

  if (bc < ac)
    h2 = myfir1(nn-1, bc/ac);
  else
    h2 = 1;
  end

  % Do convloution with separable kernels to maximize speed
  hr  = length(h1);
  hc  = length(h2);
  hrm = fix((hr-1)/2);
  hrp = fix(hr/2);
  hcm = fix((hc-1)/2);
  hcp = fix(hc/2);

  if (hr > 1)
    % Filter down each column with h1
    A1 = zeros(ar, ac);
    for j = 1 : ac
      tmp = conv(h1, A(:,j));
      A1(:,j) = tmp(hrm+1:ar+hrm);
    end
  else
    A1 = A;
  end
  if (length(h2) > 1)
    % Filter across each row with h2
    A = zeros(ar, ac);
    for i = 1 : ar
      tmp = conv(h2, A1(i,:));
      A(i,:) = tmp(hcm+1:ac+hcm);
    end
  else
    A = A1;
  end
end

if kase(1)=='n', % Nearest neighbor interpolation
  dx = ac/bc; dy = ar/br; 
  uu = (dx/2+.5):dx:ac+.5; vv = (dy/2+.5):dy:ar+.5;
elseif all(kase == 'bil') | all(kase == 'bic'),
  uu = 1:(ac-1)/(bc-1):ac; vv = 1:(ar-1)/(br-1):ar;
else
  error(['Unknown interpolation method: ',method]);
end

%
% Interpolate in blocks
%
nu = length(uu); nv = length(vv);
blk = bestblk([nv nu]);
nblks = floor([nv nu]./blk); nrem = [nv nu] - nblks.*blk;
mblocks = nblks(1); nblocks = nblks(2);
mb = blk(1); nb = blk(2);

rows = 1:blk(1); B = zeros(nv,nu);
for i=0:mblocks,
  if i==mblocks, rows = (1:nrem(1)); end
  for j=0:nblocks,
    if j==0, cols = 1:blk(2); elseif j==nblocks, cols=(1:nrem(2)); end
    if ~isempty(rows) & ~isempty(cols)
      [u,v] = meshgrid(uu(j*nb+cols),vv(i*mb+rows));
      % Interpolate points
      if kase(1) == 'n', % Nearest neighbor interpolation
        B(i*mb+rows,j*nb+cols) = interp2(A,u,v, 'nearest');
      elseif all(kase == 'bil'), % Bilinear interpolation
        B(i*mb+rows,j*nb+cols) = interp2(A,u,v,'linear');
      elseif all(kase == 'bic'), % Bicubic interpolation
        B(i*mb+rows,j*nb+cols) = interp2(A,u,v,'cubic');
      end
    end
  end
end

