function Pf = normals_to_facets(Pn)
%
% NORMALS_TO_FACETS: Pf = normals_to_facets(Pn);
%
% Convert a set of polyhedrons in normals form to facets form.  Normals form is 
% used to represent convex polyhedra as an intersection of halfspaces. Facets form 
% consists of boundary contours around each facet with points ordered so that right hand
% rule yields outward facing normals.
%
% INPUT:
% -----
% Pn - (n_cells x 1) cell array; each cell contains a normals representation for a single 
%        polyhedron; the "normals" representation models a convex polyhedron as an 
%        intersection of halfspaces.
%
% OUTPUT:
% ------
% Pf - (n_cells x 1) cell array; each cell contains a facets representation for a single
%        polyhedron; the "facets" representation consists of the contours around each
%        facet with vertices ordered so that right hand rule yields outward facing normal.
% 
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% order_vertices.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

EPS = 1e-07;

n_cells = size(Pn,1);
Pf = cell(n_cells,1);
Q = zeros(3,3);

for n = 1:n_cells
  A = Pn{n,1};
  n_facets = size(A,1); % nominally each halfspace constraint induces a facet
  F = cell(n_facets, 1);
  true_facets = 0;
  for f = 1:n_facets
    Q(1,:) = A(f,1:3);
    % Determine all possible vertices that could lie on plane f
    V = [];
    for g = 1:n_facets
      Q(2,:) = A(g,1:3);
      for h = 1:n_facets
	Q(3,:) = A(h,1:3);
        % Is Q invertible?
	if (rcond(Q) > 10*EPS)
  	  v = inv(Q)* [A(f,4); A(g,4);A(h,4)];
	
	  % Does v satisfy *all* the halfspace constraints
	  if (all(A*[v;-1] <= EPS))
	    V = [V; v'];
          end
        end
      end
    end
    
    if (size(V,1) >= 3)
      % Need to order the vertices and insure proper sense to get 
      %   outward normal from right hand rule
      W = order_vertices(V, Q(1,1:3)');
      if (size(W,1) >= 3)
        true_facets = true_facets+1;
        F{f,1} = W; % Note: start point is not repeated at the end
      else
	F{f,1} = []
      end
    else
      % If we don't find at least three vertices, this constraint is inactive
      %   and we can discard the corresponding degenerate facet.
      F{f,1} = [];
    end
  end
  


  % If any degenerate facets, remove those
  if (true_facets ~= n_facets)
    Ftmp = F;
    F = cell(true_facets,1);
    k = 0;
    for f = 1:n_facets
      if (~isempty(Ftmp{f,1}))
	k = k+1;
	F{k,1} = Ftmp{f,1};
      end
    end
  end
  Pf{n,1} = F;
end
