function W = order_vertices(V, n_hat)
%
% ORDER_VERTICES: W = order_vertices(V, n_hat);
%
% Given a set of vertices that lie on a plane in 3D, sort these
% into a proper contour order. Applying the right hand rule to
% the resulting contour will give a positive projection along
% n_hat.
%
% INPUT:
% -----
% V     -
% n_hat -
% 
% NOTE: Redundant vertices are eliminated
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% distmat.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

EPS = 1e-07;
if (nargin < 2)
  normal_align = 0;
else 
  normal_align = 1;
end

n_vertices = size(V,1);

% Center about mu and convert to unit vectors
mu = mean(V);
Vc = V - ones(n_vertices,1) * mu;
for j = 1:n_vertices
  Vc_hat(j,:) = Vc(j,:) / norm(Vc(j,:)); 
end

% create another vector in plane and perp to Vc_hat(1,:)
done = 0;
j = 2;
while (~done)
  axb = cross(Vc_hat(1,:), Vc_hat(j,:));
  if (norm(axb) > 10 * EPS)
    axb_xa = cross(axb, Vc_hat(1,:));
    done = 1;
  end
  j = j+1;
  if (j > n_vertices)
    done = 2;
  end
end
if (done == 1)
  % Successfully found a perp to Vc_hat(1,:)
  % Determine angle with respect to first vector and sort
  th = zeros(n_vertices,1);
  for j = 1:n_vertices
    cth = Vc_hat(j,:) * Vc_hat(1,:)';
    sth = Vc_hat(j,:) * axb_xa';  
    th(j,1) = atan2(sth, cth);
  end
  [th_sort, th_ind] = sort(th);
  W = V(th_ind,:);
  W = [W; W(1,:)];
else
  fprintf(1, 'WARNING (order_vertices): failed to find a perp vector - returning []\n');
  W = [];
  return
end

% Eliminate redundant vertices
keep = ones(n_vertices+1,1);
for j = 1:n_vertices
  if (distmat(W(j,:), W(j+1,:)) < EPS)
    keep(j+1,1) = 0;
  end
end
keep(n_vertices+1,1) = 0;
W = W(logical(keep), :);
n_vertices = size(W,1);


if (normal_align == 1)
  if (size(W,1) >= 3)
    % Check normal vector condition
    wvec = (cross(W(2,:)-W(1,:), W(3,:)-W(1,:)))';
    if (n_hat' * wvec < 0)
      W = flipud(W); % traverse vertices in reverse order
    end
  else
    fprintf(1, 'WARNING (order_vertices): W is degenerate\n');
  end
end


