function [I, NV] = pds_read(fname)
% PDS_READ: [I, NV] = pds_read(fname);
%
% INPUT:
% -----
% fname - filename
%
% OUTPUT:
% ------
% NV     - (N x 2) cell array of names and values from text header
% I     - image data
%
% HISTORY:
% -------
% 2005/07/22 (MCB) - Factored most of header reading stuf in pds_head so that
%                      part could be reused for reading .qub data.
%
%                      Also, found Had a problem with pds_head on some of the cassini .qub
%                      data headers, but main bug turned out to be in tokenizer.m
%                      (wasn't checking for TAB as whitespace). Also, made change
%                      pds_head so tabs between double quotes get back-filled properly.
%                     
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% pds_head.m
% get_value.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

[B, NV] = pds_head(fname);
lB = length(B);
istart = str2num(get_value(NV, '^IMAGE'));
rec_type  = get_value(NV, 'RECORD_TYPE');
if (strcmp(rec_type, 'FIXED_LENGTH') ~= 1)
  fprintf(1, 'ERROR (pds_read): RECORD_TYPE must be FIXED_LENGTH not |%s|\n', rec_type);
  I = [];
end
rec_bytes = str2num(get_value(NV, 'RECORD_BYTES'));
hend = (istart-1)*rec_bytes; % end of header stuff before image

%------------------------------------------------------------------------------------------
% Find LINES, LINE_SAMPLES, SAMPLE_TYPE, SAMPLE_BITS
nr = str2num(get_value(NV, 'LINES'));
nc = str2num(get_value(NV, 'LINE_SAMPLES'));

sample_type = get_value(NV, 'SAMPLE_TYPE');
sample_bits = str2num(get_value(NV, 'SAMPLE_BITS'));

ldat = nr*nc*sample_bits/8;
dat = B(hend+1: hend + ldat);

if (strcmp(sample_type, 'MSB_UNSIGNED_INTEGER') == 1)
  if (sample_bits == 8)
    I = dat;
  elseif (sample_bits == 16)
    I = dat(1:2:ldat) * 256 + dat(2:2:ldat);
  elseif (sample_bits == 32)
    I = dat(1:4:ldat) * 256^3 + dat(2:4:ldat)*256^2 + dat(3:4:ldat)*256+ dat(4:4:ldat);
  else
    fprintf(1, 'ERROR (pds_read): unrecognized value for sample_bits = %d\n', sample_bits);
    NV = []; I = [];
    return
  end
elseif (strcmp(sample_type, 'LSB_UNSIGNED_INTEGER') == 1)
  if (sample_bits == 8)
    I = dat;
  elseif (sample_bits == 16)
    I = dat(1:2:ldat) + dat(2:2:ldat) * 256;
  elseif (sample_bits == 32)
    I = dat(1:4:ldat) + dat(2:4:ldat)*256 + dat(3:4:ldat)*256^2+ dat(4:4:ldat)*256^3;
  else
    fprintf(1, 'ERROR (pds_read): unrecognized value for sample_bits = %d\n', sample_bits);
    NV = []; I = [];
    return
  end
else
  fprintf(1, 'ERROR (pds_read): unrecognized value for sample_type = %s\n', sample_type);
  NV = []; I = [];
  return
end

I = reshape(I, nc, nr)';
