function h = plot_edges(S, min_length, cols, lwidth)
%
% PLOT_EDGES: h = plot_edges(S, min_length, cols, lwidth);
%
% INPUT:
% -----
% S - cell array, each element is an (n X 2) matrix with points on edge
%      col1 = x, col2 = y
%
% min_length - minimum number of points in a segment
%
% cols - a string or cell array with colors to use for edges. 
%           If a cell array it must be (N X 1). Each cell contains a linecode 
%             string for the n-th edge. 
%           If a one element string, e.g., 'b', then all edges are drawn in 
%             this color. 
%           If a multi-element string, e.g., 'rgbcymk', then successive edges 
%             are drawn cycling through colors in string.
%           If equal to the one element string 'rand', you get random
%             colors for each contour chosen from rgbcmy 
%           If equal to 'trand', you get "totally random" colors 
%             (chosen from (1 X 3) RGB vectors with min(R,G,B) >= 0.3).
%
% lwidth - scalar or (N X 1) vector; line width(s) for plotting edges
%
% OUTPUT:
% ------
% h          - graphics handle of the plotted edges (OPTIONAL)
%                use this argument if you later want to delete
%                the edges from your picture (delete(h))
%
% SEE ALSO:
% --------
% edge_detection.m
%
% History:
% -------
% 20080409 (MCB) - Added a check to avoid problems with empty contours
% 20070406 (MCB) - Added option to put in a (M x 3) color matrix
% 20041025 (MCB) - Fixed the setting of line width
% 20041012 (MCB) - The cells within S now have coords in (x,y) format rather than (row, col).
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

M = size(S,1);       % Number of segments

rand_cols = 'rgbcmyk';

if (nargin < 4)
  lwidth = 1;
end

if ((size(cols, 1) == M) & (size(cols,2) == 3))
  col_method = 4;
elseif (size(cols,1) > 1)
  col_method = 1;
elseif  (strcmp(cols, 'rand') == 1)
  col_method = 2;
elseif  (strcmp(cols, 'trand') == 1)
  col_method = 3;
else
  col_method = 0;
end

hold('on');

hh = zeros(M,1);
for j = 1:M
  s = S{j,1};
  if ((~isempty(s)) & (size(s,1) >= min_length))
    x = s(:,1);
    y = s(:,2);
    if (col_method == 0)
      jj = mod(j-1, length(cols)) + 1;
      hh(j) = plot(x,y,cols(jj:jj),'Linewidth', lwidth);
    elseif (col_method == 2)
      jj = floor(6 * rand(1,1)) + 1; % dont use black
      hh(j) = plot(x,y,rand_cols(jj:jj), 'Linewidth', lwidth);     
    elseif (col_method == 3)
      jj = [0.3,0.3,0.3] + 0.7*rand(1,3);
      hh(j) = plot(x,y,'Color', jj, 'Linewidth', lwidth);     
    elseif (col_method == 4)
      hh(j) = plot(x,y,'Color', cols(j,:), 'Linewidth', lwidth);     
    else
      hh(j) = plot(x,y,cols{j,1}, 'Linewidth', lwidth);
    end
  end
end
axis('ij');

% Return graphics handles if requested
if (nargout > 0)
  h = hh;
end

