function mask = polygon_mask(imsize, X, Y)
% POLYGON_MASK: mask = polygon_mask(imsize, X, Y);
%
% Generate a "mask" from a polygon. The mask is basically a binary 
% image of size (imsize(1) X imsize(2)) in which pixels inside the
% polygon are equal to 1 and those outside the polygon are equal to 0.
%
% INPUT:
% -----
% imsize  - (2 x 1) vector specifying the size of the mask image.
% X,Y     - ((N+1) x 1) vectors specifying the X and Y coordinates
%             of the polygon vertices. The first vertex is repeated
%             at the end of the list to provide a closed figure.
%
% OUTPUT:
% ------
% mask    - (imsize(1) X imsize(2)) image.
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% clip.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

N = size(X,1) - 1; % number of vertices

nr = imsize(1); nc = imsize(2);
mask = zeros(nr, nc);

% All polygons points lie within the following bounding box:
y_lo = max(floor(min(Y)), 1);
y_hi = min(ceil(max(Y)), nr);
x_lo = max(floor(min(X)), 1);
x_hi = min(ceil(max(X)), nc);
x_range = x_hi - x_lo;

for y = y_lo : y_hi
  % Compute all intersections between the line from (x_lo,y)
  % to (x_hi, y) with the polygon lines. Only check the
  % poly lines whose y-coords straddle the current y-value
  XX = []; % x-coordinates of intersections
  for i = 1 : N
    x0 = X(i);   y0 = Y(i);
    x1 = X(i+1); y1 = Y(i+1);
    if (  ((y0 <= y) & (y1 >= y)) | ((y0 >= y) & (y1 <= y))  )
      % Check for intersection
      if (y0 == y1)
        % End-points of intersection at (x0, y) and (x1, y);
	if (~ ((x0 < 1) & (x1 < 1)) | ((x0 > nc) & (x1 > nc)) )
	  % know that at least part of intersection is within image bounds
          XX = [XX; clip(x0,1,nc); clip(x1,1,nc)];
	end
      else
        s = (y-y0)/(y1-y0);
%%        t = x0 + s * (x1-x0) - x_lo;
%%        if ((t >= 0) & (t <= x_range))
%%          % Point of intersection = (x_lo+t, y)
%%          XX = [XX; x_lo+t];
%%        end
        if ((s >= 0) & (s <= 1))
	  % Point of intersection with this segment at (x,y)
          x = x0+ s * (x1-x0);
	  XX = [XX; x];
	end
      end
    end % y-straddle
  end % polygon segments

  nx = size(XX,1);
  if (nx > 0)
    if (mod(nx,2) ~= 0)
      fprintf(1, 'WARNING (polygon_mask): odd number of intersections\n');
      keyboard
    end
    XXs = sort(XX);
    for i = 1 : 2 : nx
      if ( ~(((XXs(i) < 1) & (XXs(i+1) < 1)) | ((XXs(i) > nc) & (XXs(i+1) > nc))) )
        u0 = round(clip(XXs(i),1,nc));
        u1 = round(clip(XXs(i+1),1,nc));
        mask(y, u0:u1) = ones(1, u1-u0+1);
      end	
    end
  end
end