function P = prune_linkage(L)
%
% PRUNE_LINKAGE: P = prune_linkage(L);
%
% The incoming cell array L contains a set of 2D matrices. The (i,j) element of L{k,1}
% is the affinity between cell i in layer k-1 and cell j in layer k. This function prunes the
% linkages based on a mutual nearest neighbor criteria (a link must be both a best forward
% and a best backward match to survive.
%
% INPUT:
% -----
% L  - cell array (n_slices X 1); the element L{k,1} is a (n_cells X n_cells) matrix.
%        The (i,j) element is the affinity between cell i in layer k-1 and cell j in layer k.
%
% OUTPUT:
% ------
% P  - table of links to keep in the form k, i, j, strength
%
%---------------------------------------------------------------------------------------------

n_slices = size(L, 1);
n_cells = size(L{1,1},1);

P = zeros(n_cells*n_slices, 4);
Q = zeros(n_cells*n_slices, 4);
W = zeros(n_cells*n_slices, 4);
m = 0;
n = 0;
w = 0;
for k = 1:n_slices
  R = L{k,1};
  [hbest, jbest] = max(R');
  [vbest, ibest] = max(R);

  for i = 1:n_cells
    if (hbest(i) > 0)
      if (ibest(jbest(i)) == i)
	% Passes left-right test => keep it
	m = m+1;
	P(m, :) = [k, i, jbest(i), hbest(i)];
      end
      
      
      if (R(i, i) > 0) 
	if (jbest(i) ~= i)
	  % True link was not best forward match for node i in layer k-1
	  n = n+1;
	  Q(n, :) = [k, i, i, R(i,i)];
	elseif (ibest(jbest(i)) ~= i)
	  % True link was best forward, but not best backward match from i in layer k
	  w = w+1;
	  W(w, :) = [k, i, i, R(i,i)];
        end
      end
    end
  end
end

P = P(1:m,1:4);
Q = Q(1:n,1:4);
W = W(1:w,1:4);
keyboard
