function [V, T, C, L] = read_HxSurface(fname)
%
% read_HxSurface: [V, T, C, L] = read_HxSurface(fname);
%
% Read triangulated surface information from a text file in Amira's HxSurface format.
%
% INPUT:
% -----
% fname - in put filename
%
% OUTPUT:
% ------
% V     - (nv x 3) matrix of 3D vertices
% T     - (nt x 3) matrix of triangles (each row contains 3 indices into vertices)
% C     - (nt x 3) matrix of RGB colors for each triangle
% L     - (nt x 1) matrix of object IDS for each triangle
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% next_key_line.m
% tokenizer.m
% unquote.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

% Set to empty values initially
V = []; T = []; C = []; L = [];

fp = fopen(fname, 'r');

%---------------------------------------------------------------------------------------
% Verify first line is proper header line
buffer = fgetl(fp);
if (strcmp(buffer(1:20), '# HyperSurface ASCII') ~= 1)
  fprintf(1, 'ERROR (read_HxSurface): invalid header line = |%s|\n', buffer);
  return;
end

%---------------------------------------------------------------------------------------
% Read lines until we find Materials or reach end of file
buffer = next_key_line('Materials', fp);
if (isempty(buffer))
  fprintf(1, 'ERROR (read_HxSurface): never found Materials keyword in file %s\n', fname);
  return;
end
done = 0;
q = 0;
BLOCK = 512;
Cq = zeros(BLOCK, 3);
while (~done)
  buffer = next_key_line('color', fp);  
  if (isempty(buffer))
    done = 1;
  else
    [ns, ne] = tokenizer(buffer, '');  
    if (length(ns) ~= 4) 
      fprintf(1, 'ERROR (read_HxSurface): expected to find three color coords on line |%s|\n', buffer);    
      return;
    end
    buffer2 = next_key_line('name', fp);
    [ns2, ne2] = tokenizer(buffer2, '');
    if (length(ns2) ~= 2) 
      fprintf(1, 'ERROR (read_HxSurface): expected to find object id number in line |%s|\n', buffer);    
      return;
    end
    buffer3 = unquote(buffer2(ns2(2):ne2(2)));
    [ns3, ne3] = tokenizer(buffer3, '_');
    if (length(ns3) ~= 2)
      fprintf(1, 'ERROR (read_HxSurface): expected to find object id number in line |%s|\n', buffer);    
      return;
    end
    oid = str2num( buffer3(ns3(2):ne3(2)) );
    
    if (oid > 0)  % Skip over the background object; assume other objects are in order
      q = q+1;
      if (q > size(Cq,1))
        Cq = [Cq; zeros(BLOCK, 3)];
      end
      for j = 1:3
        Cq(q,j) = str2num( buffer(ns(1+j):ne(1+j)) );
      end
    end
  end
end
if (q == 0)
  fprintf(1, 'ERROR (read_HxSurface): did not find color information in file %s\n', fname);    
  return;
end
fseek(fp, 0, -1); % Rewind the file

%---------------------------------------------------------------------------------------
% Read lines until we find Vertices keyword or reach end of file
buffer = next_key_line('Vertices', fp);
if (isempty(buffer))
  fprintf(1, 'ERROR (read_HxSurface): never found Vertices keyword in file %s\n', fname);
  return;
end
[ns, ne] = tokenizer(buffer, '');
if (length(ns) ~= 2)
  fprintf(1, 'ERROR (read_HxSurface): expected to find number of vertices in |%s|\n', buffer)
  return;
else
  vr = str2num(buffer(ns(2):ne(2)));
end
fprintf(1, 'n_vertices = %d\n', vr);

% Read the vertex coordinates
V = zeros(vr, 3);
for i = 1:vr
  buffer = fgetl(fp);
  if (buffer == -1) 
    fprintf(1, 'ERROR (read_HxSurface): reached EOF without finding all vertex coords in file |%s|\n', fname);
    return;
  end
  [ns, ne] = tokenizer(buffer, '');  
  if (length(ns) ~= 3) 
    fprintf(1, 'ERROR (read_HxSurface): expected to find three vertex coords on line |%s|\n', buffer);    
    V = [];
    return;
  end
  for j = 1:3
    V(i,j) = str2num( buffer(ns(j):ne(j)) );
  end
end

%---------------------------------------------------------------------------------------
% Read lines until we find Patches keyword or reach end of file
buffer = next_key_line('Patches', fp);
if (isempty(buffer))
  fprintf(1, 'ERROR (read_HxSurface): never found Patches keyword in file %s\n', fname);
  V = [];
  return;
end
[ns, ne] = tokenizer(buffer, '');
if (length(ns) ~= 2)
  fprintf(1, 'ERROR (read_HxSurface): expected to find number of objects in |%s|\n', buffer);
  V = [];
  return;
else
  nq = str2num(buffer(ns(2):ne(2)));
end
fprintf(1, 'n_objects = %d\n', nq);

Lq = zeros(nq, 1); % ID label for each OBJECT
for q = 1:nq
  % Look for InnerRegion keyword
  buffer = next_key_line('InnerRegion', fp);
  if (isempty(buffer))
    fprintf(1, 'ERROR (read_HxSurface): never found InnerRegion keyword for object %d in file %s\n', q, fname);
    return;
  end
  [ns, ne] = tokenizer(buffer, '_');  
  if (length(ns) ~= 3) 
    fprintf(1, 'ERROR (read_HxSurface): expected to find InnerRegion ID number in |%s|\n', buffer);
    return;
  else
    Lq(q,1) = str2num( buffer(ns(3):ne(3)) );
  end	
  
  % Look for Triangles keyword (do not bother with OuterRegion as this is always Object_0)
  buffer = next_key_line('Triangles', fp);
  if (isempty(buffer))
    fprintf(1, 'ERROR (read_HxSurface): never found Triangles keyword for object %d in file %s\n', q, fname);
    return;
  end
  [ns, ne] = tokenizer(buffer, '');  
  if (length(ns) ~= 2) 
    fprintf(1, 'ERROR (read_HxSurface): expected to find number of triangles on line |%s|\n', buffer);    
    V = []; T = []; C = []; L = [];    
    return;
  end
  nt = str2num( buffer(ns(2):ne(2)) );
  
  Tq = zeros(nt, 3);
  for j = 1:nt
    buffer = fgetl(fp);
    if (buffer == -1) 
      fprintf(1, 'ERROR (read_HxSurface): reached EOF without finding all vertex coords in file |%s|\n', fname);
      V = []; T = []; C = []; L = [];      
      return;
    end
    [ns, ne] = tokenizer(buffer, '');  
    if (length(ns) ~= 3) 
      fprintf(1, 'ERROR (read_HxSurface): expected to find three vertex coords on line |%s|\n', buffer);    
      V = []; T = []; C = []; L = [];
      return;
    end
    for k = 1:3
      Tq(j,k) = str2num( buffer(ns(k):ne(k)) );
    end
  end
  L = [L; ones(nt,1) * Lq(q,1)];
  C = [C; ones(nt,1) * Cq(q,:)];
  T = [T; Tq];
end