function [gamma, alpha, svecs, b] = read_libsvm(filename)
% READ_LIBSVM: [gamma, alpha, svecs, b] = read_libsvm(filename);
%
% Read support vector machine models produced by libsvm.
%
% INPUT:
% -----
% filename - name of SVM file produced by libsvm
%
% OUTPUT:
% ------
% gamma - scalar;  gamma value in RBF kernel
% alpha - (n_sv X 1); signed weighting coefficients in SVM series
% svecs - (n_sv X d); each row contains a support vector
% b     - scalar; offset
%
% NOTE:
% ----
% Decision is as follows:
%   g(x) = sum_{i=1}^{n_sv} alpha(i) K(x,svecs(i,:)') - b
%
% (MCB) - Not absolutely sure on sign for b (whether it should be added or subtracted)
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

fp = fopen(filename, 'r');

% Read header info
gamma = 0;
kernel_type = -1;
for n = 1:9
  line = fgetl(fp);
  fprintf(1, '%d |%s|\n', n, line);
  L = length(line);
  if (strcmp(line, 'SV') == 1)
    break;
  end
  if (n == 2)
    kernel_type_string = line(2+length('kernel_type'):L);
    if (strcmp(kernel_type_string, 'rbf') == 1)
      kernel_type = 0;
    elseif (strcmp(kernel_type_string, 'linear') == 1)
      kernel_type = 1;
    else
      fprint(1, 'ERROR (read_libsvm): unsupported kernel_type = %s\n', kernel_type_string);
      gamma = []; alpha = []; svec =[]; b = [];
      return
    end
  end
  if (kernel_type == 0) % RBF
    if (n == 3)
      % Get gamma value
      gamma = str2num(line(2+length('gamma'):L));
    end
    if (n == 5)
      % Get toal number of support vectors
      nsv = str2num(line(2+length('total_sv'):L));
    end
    if (n == 6)
      % Get rho (b)
      b = str2num(line(2+length('rho'):L));
    end
  elseif (kernel_type == 1)
    if (n == 4)
      % Get toal number of support vectors
      nsv = str2num(line(2+length('total_sv'):L));
    end
    if (n == 5)
      % Get rho (b)
      b = str2num(line(2+length('rho'):L));
    end
  end
end

if (kernel_type == 0)
  st = 10;
elseif (kernel_type == 1)
  st = 9;
end

alpha = zeros(nsv,1);
for n = st:(st+nsv-1)
  fprintf(1, '%d ', n);
  line = fgetl(fp);
  [s,e] = tokenizer(line, ' :');
  if (n == st)
    nf = (length(s)-1)/2;
    svecs = zeros(nsv, nf);
  end
  alpha(n,1) = str2num(line(s(1):e(1)));

  for f = 1:nf
    svecs(n,f) = str2num(line(s(1+2*f):e(1+2*f)));
  end
end
fclose(fp);
