function indset = salient_points(Kset, w, thr);
%
% SALIENT_POINTS: indset = salient_points(Kset, w, thr);
%
% Determine salient points of a set of 1D profiles contained in the
% cell array Kset. The salient points are the local maxima and minima 
% over windows of width w. An absolute threshold may also be applied.
%
% INPUT:
% -----
% Kset  - (N X 1) cell array; each cell contains a (column) vector
% w     - window over which profile value must be a local extrema
% thr   - (2 X 1) vector containing lower and upper thresholds on f
%         [low_thr; hi_thr]
%
% OUTPUT:
% ------
% indset - (N X 1) cell array; each cell is a vector of salient point indices
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% slide_win.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

if (nargin < 3)
  thr = [Inf; -Inf];
end

N = size(Kset,1); % number of profiles in the cell array
indset = cell(N,1);

% Determine local maxima and minima
for n = 1:N
  k = Kset{n,1};
  G  = slide_win(k, w, 'xtrp'); % G will be (n X w)
%  G  = slide_win(k, w, 'hold'); % G will be (n X w)
  ind_maxima  = ((k >= thr(2)) & (k == max(G')'));  % Find local max above threshold
  ind_minima  = ((k <= thr(1)) & (k == min(G')'));  % Find local min below trheshold
  ind_extrema = find(ind_maxima | ind_minima);    % Local extrema
  indset{n,1} = ind_extrema;
end


