function [L_final, C_final, Cset_final] = segmentation_editor(I, L, C, Cset)
%
% SEGMENTATION_EDITOR: [L_final, C_final, Cset_final] = segmentation_editor(I, L, C, Cset);
%
% Manual editing of segmentation results.
%
% INPUT:
% -----
% I     - (nr X nc X nd) raw image (grayscale or color)
% L     - (nr X nc) label image with values from 0 (background) to nobj
% C     - (nobj X 3) color table
% Cset  - (nobj X 1) cell array with boundary contour of each segment.
%
% OUTPUT:
% ------
% L_final    - (nr X nc) new label image with values form 0 (background) to nobj_new
% C_final    - (nobj_new X 3) new color table
% Cset_final - (nobj_new X 1) cell array with boundary contours of each segment.
%
% NOTE:
% ----
% Three figure windows are used: (1) the first shows the raw image with boundary contours,
% (2) the second shows the segmentation image (i.e., the label image using the color map), and
% (3) the third shows the nuclues channel only (assumed to be the green channel).
%
% When the x-axis label of the first figure says "Wait Mode', you can click MB1 (mouse button 1)
% to enter drawing mode and draw a polygon around a nucleus that should have been marked, but
% wasn't. With MB2 you can enter 'Keyboard Mode' which puts you at a keyboard prompt (typically you
% will not use this one; type return at the command line prompt to get out of keyboard mode). 
% With MB3 you can enter 'Delete Mode', deleting the object that you clicked on (or near - 
% there is a 2 pixel allowance if you are near but not exactly on an object; this is helpful 
% for deleting very small object which may be hard to click on exactly). The first two figures
% will be updated after completion of each editing act (adding or deleting). If two nuclei are
% improperly joined together, you will first need to delete and then draw the two separately.
%
% The add function does not like it if you draw nuclei overlapping so definitely avoid this!
% If you have non-zero overlap, you will get dumped to a keyboard prompt. You may want to
% say delete_polygon(Z); return; and then retry to draw without overlap.
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% ascale.m
% hon.m
% hof.m
% GC2.m
% icluster_display.m
% draw_polygon.m
% polygon_mask.m
% bitmap_contours.m
% icluster_keep.m
% icluster_contours.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

[nr, nc, nd] = size(I);
nobj = size(C,1);

ftab = zeros(3,1); % table of figure numbers
htab = zeros(nobj,1);

for k = 1:3
  ftab(k,1) = figure;
end

% Display raw image with contours in ftab(1,1)
figure(ftab(1,1));
if (nd == 3)
  image(I); hon;
else
  image(ascale(I)); GC2; hon;
end

for i = 1:nobj
  c = Cset{i,1};
  htab(i,1) = plot([c(:,1); c(1,1)], [c(:,2); c(1,2)], 'Color', 'b');
end
keep = ones(nobj, 1);

% Display segmentation image in ftab(2,1)
figure(ftab(2,1));
icluster_display(L, C); hon;

% Display nucleus channel in ftab(3,1)
figure(ftab(3,1));
image(ascale(squeeze(I(:,:,2)))); GC2;

%---------------------------------------------------------------------------------
% MAIN LOOP
done = 0;
while (~done)
  figure(ftab(1,1));
  xlabel('wait mode', 'Fontsize', 16);  
  [x, y, b] = ginput(1);
  if (b == 1)
    xlabel('add mode', 'Fontsize', 16);
    Z = draw_polygon('b+', 'y-');
    M = polygon_mask([nr, nc], Z(:,1), Z(:,2));
    overlap = sum(sum((M > 0) .* (L > 0)));
    if (overlap == 0)
      nobj = nobj + 1;
      htab = [htab; 0];
      keep = [keep; 1];
      L = L + nobj * (M > 0);
      C = [C; randcol(1, 0.3)];
      
      % Update displayed segmentation image
      figure(ftab(2,1));
      hof;
      icluster_display(L, C);
      hon;
      figure(ftab(1,1));
      
      c = bitmap_contours((M > 0));
      % Add contour on display raw image
      delete_polygon(Z);
      htab(nobj,1) = plot([c(:,1); c(1,1)], [c(:,2); c(1,2)], 'Color', 'c');
    else
      fprintf(1, 'Non-zero overlap\n');
      keyboard
    end
  elseif (b == 2)
    xlabel('keyboard mode', 'Fontsize', 16);
    keyboard
  elseif (b == 3)
    xlabel('delete mode', 'Fontsize', 16);    
    % Determine which segment was selected
    xx = round(x);
    yy = round(y);
    hit = 0;
    if ((xx >= 1) & (xx <= nc) & (yy>=1) & (yy <= nr))
      tgt = L(yy, xx);
      if (tgt ~= 0)
	hit = 1;
      else
	d_best = Inf;
	for dy = -2:2
	  vv = yy+dy;
	  for dx = -2:2
	    uu = xx+dx;
            if ((uu >= 1) & (uu <= nc) & (vv>=1) & (vv <= nr))	    
	      if (L(vv,uu) ~= 0)
	        hit = 1;
	        d = abs(dx)+abs(dy);
	        if (d < d_best)
	  	  d_best = d;
		  tgt = L(vv,uu);
	        end
	      end
	    end
	  end
	end
      end
    end
    if (hit == 1)
      xlabel(['delete target = ',int2str(tgt)], 'Fontsize', 16);    	
      keep(tgt,1) = 0;
      [i,j] = find(L == tgt);
      for k = 1:length(i)
        L(i(k), j(k)) = 0;
      end

      % Update displayed segmentation image
      figure(ftab(2,1));
      hof;
      icluster_display(L, C);
      hon;
      figure(ftab(1,1));

      % Remove contour from raw image
      delete(htab(tgt,1));
    end
  else
    fprintf(1, 'ERROR: unrecognized button mode\n');
  end
end

%---------------------------------------------------------------------------------
% FINALIZE
ind = find(keep);
[L_final, nobj_final] = icluster_keep(L, nobj, ind);
C_final = C(ind, :);

if (nargout > 2)
  Cset_final = icluster_contours(L_final, nobj_final);
end

keyboard
