function [Lfinal, Cfinal] = sepal_nuclei(iname, intensity_thr)
%
% SEPAL_NUCELI: [Lfinal, Cfinal] = sepal_nuclei(iname, intensity_thr);
%
% Segment out nuclei in projection images.
%
% INPUT:
% -----
% iname         - full image name including path
% intensity_thr - intensity threshold applied to median filtered image (typically a value of 
%                   40 to 50 works well).
%
% OUTPUT:
% ------
% Lfinal - label image; same size as the original image. Each pixel is given an integer
%            ID number corresponding to the nucleus it belongs with (or 0 for background)
% Cfinal - (nobj X 3) array of colors
%
% NOTE:
% ----
% Uses Matlab's built-in watershed function.
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% icluster.m
% median_filter.m
% icluster_contours.m
% icluster_moments.m
% clip.m
% local_extrema.m
% icluster_elim.m
% randcol.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------
rho_thr = 0.6;
if (nargin < 2)
  intensity_thr = 40;
end

I = imread(iname);
nr = size(I,1);
nc = size(I,2);
nd = size(I,3);

if (nd == 1)
  % input image is grayscale; assume it is nucleus channel
  J = I;
elseif (nd == 3)
  % input image is color; assume nuclei are in green channel
  J = squeeze(I(:,:,2));
else
  fprintf(1, 'ERROR (sepal_nuclei): incoming image must be grayscale or RGB\n');
  Lfinal = [];
  Cfinal = [];
  return
end
Y = median_filter(J, 3);

% Find local maxima in Y
[x, y, t, v] = local_extrema(Y, 5, 1);

% Consolidate local maxima
Q = zeros(nr, nc);
for k = 1:length(x)
  if (v(k) > intensity_thr)
    Q(y(k), x(k)) = 1;
  end
end
[LQ, NQ] = icluster(Q, [1;1]);
AQ = icluster_moments(LQ, NQ, 0.1);
xx = clip(round(AQ(:,2)), 1, nc); % Consolidated x-coordinates for local maxima
yy = clip(round(AQ(:,3)), 1, nr); % Consolidated y-coordinates for local maxima
QQ = zeros(nr, nc);
for k = 1:length(xx)
  QQ(yy(k), xx(k)) = 1;
end
  
% Find connected components from pixels in Y that are above threshold
[L, nobj] = icluster((Y > intensity_thr), [1;1]);

% Extract moments from each of the components
A = icluster_moments(L, nobj, 0.1);

% Determine how many local maxima are contained in each connected component
cnt = zeros(nobj,1);
for k = 1:nobj
  cnt(k,1) = sum(sum(  (L == k) & (QQ == 1) ));
end

% Extract bounding contour for each connected component
Cset = icluster_contours(L, nobj);

% Check indentation of contour from estimated ellipse
rho = zeros(nobj,1);
for k = 1:nobj
  mu=A(k,2:3)';
  S=4*[A(k,4), A(k,5); A(k,5), A(k,6)]; 
  invS = inv(S);
  P = Cset{k,1};
  n=size(P,1);
  q = zeros(n,1);
  for i=1:n, 
    q(i,1) = (P(i,:)-mu') * invS * (P(i,:)'-mu);
  end
  rho(k,1) = sqrt(min(q));
end

% Determine which regions need splitting, i.e., regions with mutliple disjoint local
%   maxima and significant indentation from the estimated boundary ellipse (from region moments).
need_split = find( (rho < rho_thr) & (cnt >= 2) );
C = ones(nobj, 1) * [0, 0, 1];          % display most things in blue
C(need_split,:) = ones(length(need_split),1)*[1,0,0]; % display those in need of splitting in red

not_need_split = find(~((rho < rho_thr) & (cnt >= 2)) );
[Lnew, nobj_new] = icluster_elim(L, nobj, not_need_split); % eliminate clusters not needing split so
                                                           %   Lnew is a new label image containing only 
							   %   the regions that do need splitting
[yyy, xxx] = find((QQ == 1) & (Lnew > 0)); % local maxima within the regions needing splitting

%%%% Plot to show connected components and local maxima
%%% icluster_display(L, C);
%%% hon;
%%% plot(xxx, yyy, 'w+');

Ynew = (Lnew > 0) .* Y; % Create new intensity image covering the regions needing splitting
W = watershed(256-Ynew); % Compute catchment basins of inverted image

%% Convert watershed pixels into catchment basin labels
%% Slow and not really necessary???
%Wnew = W;
%for i=2:nr-1
%  for j = 2:nc-1
%    if (Wnew(i,j) == 0)
%      N = neighbors([i, j], [nr, nc], 8);
%      min_nonz = Inf;
%      for k = 1:length(N,1)
%	v = Wnew(i+N(k,1), j+N(k,2));
%	if ((v > 0) & (v < min_nonz))
%	  min_nonz = v;
%	end
%      end
%      W(i,j) = min_nonz;
%    end
%  end
%end

W = W .* (Lnew > 0); % Mask catchment basins with regions that needed splitting

% Combine catchment basins with regions that did not require splitting
[Lpure, nobj_pure] = icluster_elim(L, nobj, need_split);
Lfinal = Lpure + (W + nobj_pure * ones(nr,nc) .* (W > 0)); 
nobj_final = max(max(Lfinal));
Cfinal = randcol(nobj_final, 0.3);

%keyboard
