function F = slide_win(f, w, bound_flag);
%
% SLIDE_WIN: F = slide_win(f, w, bound_flag);
%
% Assume f is a column vector, then F is a matrix in which
% row i is a w pixel window around pixel i in f. What to
% do at the boundaries can be specified by bound_flag (DEFAULT
% is to pad with zeros.
%
% INPUT:
% -----
% f        - (n X 1) input vector
% w        - window size (*must* be odd)
% bound_flag - method of handling ends of f. Options are as follows:
%
%            'zero' - pad with zeros
%            'hold' - repeat first and last values of f
%            'wrap' - circularly wrap
%            'xtrp' - extrapolate values (using spline)
%
% OUTPUT:
% ------
% F        - (n X w) matrix; row i j of F contains a length w window around
%              point i in f.
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

% Determine boundary method or set to DEFAULT
if (nargin < 3)
  bnd = 'zero';
else
  bnd = bound_flag(1:4);
end

n = length(f);
ww = floor((w-1)/2);

% Pad f using the appropriate boundary method
if (strcmp(bnd,'zero'))
  ff = [zeros(ww,1); f; zeros(ww,1)];
elseif (strcmp(bnd,'hold'))
  ff = [f(1) * ones(ww,1); f; f(n) * ones(ww,1)];
elseif (strcmp(bnd,'wrap'))
  ff = [f(n-ww+1:n); f; f(1:ww)];
elseif (strcmp(bnd,'xtrp'))
  xold = (1:n)';
  xnew = (-ww+1:n+ww)';
  ff = interp1(xold,f,xnew,'spline');
else
  error('Unsupported boundary method');
end

F = ff * ones(1, w);
N = size(F,1);
for j = 2 : w
  F(:,j) = [F(j:N, 1); F(1:j-1, 1)];
end
 
F = F(1:n,:);

