function [V, T, C, L] = triangulate_cell_objects(B)
% TRIANGULATE_CELL_OBJECTS: [V, T, C, L] = triangulate_cell_objects(B)
%
% Given a cell array in which each cell has the 3D contours (from various stack levels)
% of a particular object, we construct a triangulated surface model for each object.
%
% INPUT:
% -----
% B - cell array with n_objects cells. Cell k contains a set of 3D
%       contours corresponding to object k.
%
% OUTPUT:
% ------
% V - (nv x 3) matrix of 3D vertices
% T - (nt x 3) matrix of triangles (each row contains 3 indices into V)
%       Note: that the sense of the triangle must be in correct order
%       to insure surface normals are pointing in the right direction
% C - (nt x 3) matrix of RGB colors for each triangle
% L - (nt x 1) matrix with object label for each triangle
%
% NOTE:
% ----
% Microscope coordinate system has X-axis going to the right, y-axis 
%   going down, and z-axis going into the screen (standard camera system).
%
% SEE ALSO:
% --------
% write_HxSurface.m
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% contour_distance.m
% randcol.m
% curvature.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

M = 20; % Number of points for resampling contours
K_thr = 0;
n_objects = size(B,1);

V = [];
T = [];
C = [];
L = [];

for k = 1:n_objects
  Vk = [];     % vertices for object k
  Tk = [];     % triangles for object k
  S = B{k,1};  % set of contours for object k
  
  % assume contours are already sorted by level (smaller Z to bigger Z)
  %   need to resample and match up contours on adjacent levels; 
  %   Make sure points in top contour are ordered so that surface
  %   normal points in -Z direction.
  n = size(S,1);
  [d, U1, U2] = contour_distance(S{1,1}(:,1:2), S{1,1}(:,1:2), M);
  z1 = S{1,1}(1,3); % z value for this first contour
  K = curvature(U2);
%  % Calculate total arc length  
%  dx = diff(U2(:,1));
%  dy = diff(U2(:,2));
%  s = sum(sqrt(dx.^2 + dy.^2));
  if (sum(K) > K_thr)  % Total turning angle
    U1 = flipud(U1); % reverse sense of contour
    U2 = flipud(U2); % reverse sense of contour
  end
  Vk = [U1, z1*ones(M,1)];
  offset = 0;
  
  for j = 1:n-1
    [d, U1, U2] = contour_distance(U2, S{j+1,1}(:,1:2), M);
    z2 = S{j+1,1}(1,3); % z-value for next contour
    Vk = [Vk; [U2, z2*ones(M,1)]];
    T1 = offset*M + [(1:M)', ((M+1):(2*M))', [(2:M)'; 1]];
    T2 = offset*M + [[(2:M)';1], ((M+1):2*M)', [((M+2):2*M)'; M+1]];
    Tk = [Tk; T1; T2];	
    offset = offset+1;
  end
  
  r = size(Vk,1);
  
  % Add triangulation of top (smaller z) and bottom (bigger z) layer
  %   Implicitly assuming these contours satisfy "star property"
  mu_top = mean(S{1,1});
  Vk = [Vk; mu_top];
  T3 = [(r+1) * ones(M,1), (1:M)', [(2:M)'; 1]];
  
  mu_bot = mean(S{n,1});
  Vk = [Vk; mu_bot];
  T4 = [[(r-M+2:r)'; r-M+1], (r-M+1:r)', (r+2) * ones(M,1)];

  Tk = [Tk; T3; T4];
  ntk = size(Tk,1);
  
  vr = size(V,1);
  V = [V; Vk];
  T = [T; (vr*ones(ntk,3)+Tk)];
  col = randcol(1, 0.3);
  C = [C; ones(ntk,1) * col];
  L = [L; ones(ntk,1) * k];  
end
