function [v, xyz, l] = volumes_from_triangulated(V, T, L, S)
%
% VOLUMES_FROM_TRIANGULATED: [v, xyz, l] = volumes_from_triangulated(V, T, L, S);
%
% Calculate the volume for each object in a set of triangulated objects.
%
% INPUT:
% -----
% V - (n_vertices X 3) matrix of vertices
% T - (n_triangles X 3) matrix of triangles
% L - (n_triangles X 1) matrix of object ID labels (one for each triangle)
% S - (3 X 1) vector of scale factors for x,y, and z dimensions
%
% OUTPUT:
% ------
% v   - (n_objects x 1) vector of volumes
% xyz - (n_objects X 3) matrix of centroids 
% l   - (n_objects X 1) vector of object ID labels (one for each object)
%
% NOTE:
% ----
% We assume that objects are star convex with respect to their centroids.
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

% Construct diagonal scaling matrix
if (nargin < 4)
  D = eye(3,3);
else
  D = diag(S);
end

l = unique(L);
n_objects = length(l);
v = zeros(n_objects, 1);
H = zeros(3,3);

for n = 1: n_objects
  % Find all the triangles for this object
  T_ind = find(L == l(n));
  if (~isempty(T_ind))
    Tn = T(T_ind, :);
  
    % Find all the vertices participating in this object
    V_ind = unique(Tn(:));
    
    % Find centroid
    V_mu = mean(V(V_ind,:)); % leave as a row vector
    xyz(n,:) = V_mu;
    
    % Connect each triangle to centroid to form a tetrahedron
    for i = 1:length(T_ind)
      Tni = T(T_ind(i), :);
      H(1,:) = V(Tni(1,1),:) - V_mu;
      H(2,:) = V(Tni(1,2),:) - V_mu;
      H(3,:) = V(Tni(1,3),:) - V_mu;      
      H = H * D;
      v(n,1) = v(n,1) + det(H)/6;
    end
  end
end