function write_HxSurface(fname, V, T, C, L)
%
% write_HxSurface: write_HxSurface(fname, V, T, C, L);
%
% Write triangulated surface information to a text file in Amira's HxSurface format.
%
% INPUT:
% -----
% fname - output filename
% V     - (nv x 3) matrix of 3D vertices
% T     - (nt x 3) matrix of triangles (each row contains 3 indices into vertices)
% C     - (nt x 3) matrix of RGB colors for each triangle
% L     - (nt x 1) matrix of object IDS
%
% OUTPUT:
% ------
% a text file in Amira's HxSurface format is written to fname
%
% NOTE:
% ----
% We assume all triangles corresponding to same object ID have the
% same color, i.e., one entry under Materials for each object ID.
% Also, we use one "patch" for each object ID.
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

[vr, vc] = size(V);
[tr, tc] = size(T);
[cr, cc] = size(C);
[lr, lc] = size(L);

if ((vc ~= 3) | (tc ~= 3) | (cc ~= 3))
  fprintf(1, 'ERROR(write_HxSurface): V, T, and C matrices should have three columns\n');
  return
end

if ((tr ~=  cr) | (tr ~= lr))
  fprintf(1, 'ERROR(write_HxSurface): T, C, and L matrices should have same number of rows\n');
  return
end

fp = fopen(fname, 'w');
if (fp == -1)
  fprintf(1, 'ERROR(write_HxSurface): error opening file %s\n', fname);  
  return
end

% Determine number of unique object IDs
[Lq, Iq, Jq] = unique(L);
nq = size(Lq, 1);

% Header
fprintf(fp, '# HyperSurface ASCII\n\n');

% Parameters
fprintf(fp, 'Parameters {\n');
fprintf(fp, '}\n\n');

% Materials
fprintf(fp, 'Materials {\n');

% Background material (?)
fprintf(fp, '  {\n');
fprintf(fp, '    color 0 0 0 \n');
fprintf(fp, '    name \"Object_0\"\n');  
fprintf(fp, '  }\n');  

% Colored material for each object
for q = 1:nq
  fprintf(fp, '  {\n');
  fprintf(fp, '    color %f %f %f\n', C(Iq(q,1),1), C(Iq(q,1),2), C(Iq(q,1),3));
  fprintf(fp, '    name \"Object_%d\"\n', Lq(q,1));  
  fprintf(fp, '  }\n');  
end
fprintf(fp, '}\n\n');

% Vertices
fprintf(fp, 'Vertices %d\n', vr);
for i=1:vr
  fprintf(fp, '    %f %f %f\n', V(i,1), V(i,2), V(i,3));
end
fprintf(fp, '\n');

% Patches
fprintf(fp, 'Patches %d\n', nq);
for q = 1:nq
  fprintf(fp, '{\n');
  fprintf(fp, '    InnerRegion Object_%d\n', Lq(q,1));
  fprintf(fp, '    OuterRegion Object_0\n');  
  ind = find(L == Lq(q,1)); % which triangles belong to this object
  fprintf(fp, '    Triangles %d\n', length(ind));
  for j=1:length(ind)
    fprintf(fp, '      %f %f %f\n', T(ind(j),1), T(ind(j),2), T(ind(j),3));
  end
 fprintf(fp, '}\n');
end
fclose(fp);