function [L, num_objects] = zlink(F, link_thr)
%
% ZLINK: [L, num_objects] = zlink(F, link_thr)
%
% Assign object numbers to contours in F.
%
% INPUT:
% -----
% F        - cell array with n_levels cells; each cell contains a set of contours
%              S that were detected at a given level in the image stack; each set S
%              contains a number of contours, which are represented with an (n x 2)
%              matrix of vertices.
% link_thr - threshold on conotur distance for linking two contours
%
% OUTPUT
% ------
% L - cell array with n_levels cells; each cell contains a vector whose length
%       is equal to the number of contours at this level. The vector elements
%       contain object id numbers.
% num_objects - total number of objects found.
%
% NOTE:
% ----
% 1. Similar to connected components except we don't have to worry about label equivalences
% 2. May also want to try an area of overlap criteria for linking.
%
%---------------------------------------------------------------------------------------
% BEGIN CDS DEPENDENCIES
% bbox_test.m
% contour_distance.m
% END CDS DEPENDENCIES
%---------------------------------------------------------------------------------------

if (nargin < 2)
  link_thr = 6;
end

slop = link_thr * [-1,-1;1,1];  % slop for comparing bounding boxes

n_levels = size(F,1);

% Set up data structure for output result
L = cell(n_levels, 1);
n_contours = zeros(n_levels, 1);
for l = 1:n_levels
  S = F{l,1};
  n_contours(l,1) = size(S,1);
  L{l,1} = zeros(n_contours(l,1),1);
end


new_label = 0;
for l = 1:n_levels
  S1 = F{l,1};
  for k = 1:n_contours(l,1)
    if (L{l,1}(k,1) == 0)
      % If contour k is unlabeled give it new label
      new_label = new_label+1;
      L{l,1}(k,1) = new_label;
    end
    
    if (l < n_levels)
      C1 = S1{k,1};
      bbox1 = [min(C1(:,1)), min(C1(:,2)); max(C1(:,1)), max(C1(:,2))];
      % Check next level down to see if there is a matching contour
      S2 = F{l+1,1};
      d = zeros(n_contours(l+1,1), 1);
      for m = 1:n_contours(l+1,1)
	C2 = S2{m,1};
        bbox2 = [min(C2(:,1)), min(C2(:,2));  max(C2(:,1)), max(C2(:,2))];	
	if (bbox_test(bbox1+slop, bbox2+slop) == 1)
 	  d(m,1) = contour_distance(C1, C2);
	else
	  d(m,1) = Inf;
	end
      end
      [mind, indd] = min(d);
      if (mind < link_thr)
	% Contour indd(1) at level l+1 matches contour k at level l
	L{l+1,1}(indd(1),1) = L{l,1}(k,1);
      end
    end
  end
end
num_objects = new_label;